import os
import pathlib
import re
import shutil
import tarfile
import fnmatch

from itertools import groupby
from eatb import VersionDirFormat
from eatb.checksum import check_transfer
from eatb.ipstate import IpState
from eatb.utils.fileutils import rec_find_files, fsize, FileBinaryDataChunks
from eatb.utils.fileutils import to_safe_filename, list_files_in_dir, copy_file_with_base_directory
from pairtree import PairtreeStorageFactory, ObjectNotFoundException
from eatb import logger
from eatb.checksum import files_identical
from eatb.packaging import ChunkedTarEntryReader
from eatb.utils.reporters import default_reporter


class PairtreeStorage:
    """
    Pairtree storage class allowing to build a filesystem hierarchy for holding objects that are located by mapping
    identifier strings to object directory (or folder) paths with two characters at a time.
    """
    storage_factory = None
    repository_storage_dir = None

    def __init__(self, repository_storage_dir, representations_directory="representations"):
        """
        Constructor initialises pairtree repository
        :param repository_storage_dir: repository storage directory
        :param representations_directory: representations directory
        """
        #super().__init__(repository_storage_dir)
        self.storage_factory = PairtreeStorageFactory()
        self.repository_storage_dir = repository_storage_dir
        self.representations_directory = representations_directory
        self.repo_storage_client = self.storage_factory.get_store(store_dir=repository_storage_dir, uri_base="http://")
        self.representations_directory = representations_directory

    # noinspection PyProtectedMember
    def store(self, identifier, source_file_path, progress_reporter=default_reporter):
        """
        Storing a directory in the pairtree path according to the given identifier. If a version of the object exists,
        a new version is created.
        :param identifier: identifier
        :param source_file_path: source file path
        :param progress_reporter: progress reporter
        :return:
        """
        # pairtree object path
        pairtree_object_path = self.repo_storage_client._id_to_dirpath(identifier)
        # next version string
        next_version = self._next_version(identifier)
        # data directory
        target_data_directory = os.path.join(pairtree_object_path, "data")
        # create data directory path
        pathlib.Path(target_data_directory).mkdir(parents=True, exist_ok=True)
        # data version directory
        target_data_version_directory = os.path.join(target_data_directory, next_version)
        # create data version directory
        os.makedirs(target_data_version_directory, exist_ok=True)

        # archive file name
        safe_identifier_name = to_safe_filename(identifier)
        archive_file = "%s.tar" % safe_identifier_name
        # check source file exists
        if not os.path.exists(source_file_path):
            raise ValueError("Source file does not exist: %s" % source_file_path)
        # target file path
        target_file_path = os.path.join(target_data_version_directory, archive_file)
        # copy file
        shutil.copy2(source_file_path, target_file_path)
        progress_reporter(100)
        return next_version


    def get_bag_name(self, identifier: str, version: str, bagnr: int):
        safe_identifier_name = to_safe_filename(identifier)
        bag_dir_name = "b%05d" % int(bagnr)
        bag_name = "%s_%s_%s" % (safe_identifier_name, version, bag_dir_name)
        return bag_name

    def identifier_object_exists(self, identifier):
        """
        Verify if an object of the given identifier exists in the repository

        @type       identifier: string
        @param      identifier: Identifier
        @rtype:     boolean
        @return:    True if the object exists, false otherwise
        """
        logger.debug("Looking for object at path: %s/data" % (self.repo_storage_client._id_to_dirpath(identifier)))
        return self.repo_storage_client.exists(identifier, os.path.join("data"))

    def identifier_version_object_exists(self, identifier, version_num):
        """
        Verify if the given version of the object exists in the repository
        :param identifier: identifier
        :param version_num: version number
        :return:
        """
        version = VersionDirFormat % version_num
        return self.repo_storage_client.exists(identifier, "data/%s" % version)

    def _get_version_parts(self, identifier):
        """
        Get version directories
        :param identifier: identifier
        :return: version parts
        """
        return self.repo_storage_client.list_parts(identifier, "data")

    def _next_version(self, identifier):
        """
        Get next formatted version directory name
        :param identifier: identifier
        :return: next formatted version directory name
        """
        if not self.identifier_object_exists(identifier):
            return VersionDirFormat % 0
        version_num = 1
        while self.identifier_version_object_exists(identifier, version_num):
            version_num += 1
        return VersionDirFormat % version_num

    def curr_version(self, identifier):
        """
        Get current formatted version directory name
        :param identifier: identifier
        :return: current formatted version directory name
        """
        curr_version_num = self.curr_version_num(identifier)
        return VersionDirFormat % curr_version_num

    def curr_version_num(self, identifier):
        """
        Get current version number
        :param identifier: identifier
        :return: current version number
        """
        if not self.identifier_object_exists(identifier):
            return -1
        version_num = 0
        while self.identifier_version_object_exists(identifier, version_num):
            version_num += 1
        version_num -= 1
        return version_num

    def get_object_path(self, identifier):
        """
        Get absolute file path of the stored object. If the version number is omitted, the path of the highest version
        number is returned.
        :param identifier: identifier
        :param version_num: version number
        :return: absolute file path of the stored object
        """
        curr_version = self.curr_version(identifier)
        dirpath = self.repo_storage_client._id_to_dirpath(identifier)
        target_data_directory = os.path.join(dirpath, "data")
        target_data_version_directory = os.path.join(target_data_directory, curr_version)
        path_to_object = os.path.join(target_data_version_directory, "%s.tar" % to_safe_filename(identifier))
        if os.path.exists(path_to_object):
            logger.debug("Package file found at: %s" % path_to_object)
            return path_to_object
        raise ObjectNotFoundException("Package file not found")

    def get_chunked_tar_entry_reader(self, identifier: str) -> ChunkedTarEntryReader:
        tar_file_path = os.path.join(self.get_object_path(identifier), "%s.tar" % to_safe_filename(identifier))
        tar_file = tarfile.open(tar_file_path, 'r')
        return ChunkedTarEntryReader(tar_file)

    # noinspection PyProtectedMember
    def latest_version_ip_list(self) -> list:
        """
        Get a list of latest version packages from repository storage.
        :return: list of latest version packages
        """
        files = rec_find_files(self.repository_storage_dir)
        sortkeyfn = lambda s: s[1]
        tuples = []
        for repofile in files:
            if repofile.endswith(".tar"):
                f, _ = os.path.split(repofile)
                version = re.search(r'v[0-9]{5,5}', f).group(0)
                repoitem = (repofile, int(re.search(r'\d+', version).group(0)))
                tuples.append(repoitem)
        tuples.sort(key=sortkeyfn, reverse=True)
        items_grouped_by_version = []
        for key, valuesiter in groupby(tuples, key=sortkeyfn):
            items_grouped_by_version.append(dict(version=key, items=list(v[0] for v in valuesiter)))
        lastversionfiles = []
        for version_items in items_grouped_by_version:
            for item in version_items['items']:
                p, f = os.path.split(item)
                p2 = os.path.join(self.repository_storage_dir, p[:p.find("/data/v")])
                obj_id = self.repo_storage_client._get_id_from_dirpath(p2)
                if obj_id not in [x['id'] for x in lastversionfiles]:
                    lastversionfiles.append({"id": obj_id, "version": version_items['version'], "path": item})
        return lastversionfiles


    # noinspection PyProtectedMember
    def get_dir_path_from_id(self, identifier):
        """
        Get directory path from id
        :param identifier: identifier
        :return: directory path
        """
        return self.repo_storage_client._id_to_dirpath(identifier)

    def get_tar_file_path(self, identifier):
        curr_version = self.curr_version(identifier)
        dirpath = self.repo_storage_client._id_to_dirpath(identifier)
        target_data_directory = os.path.join(dirpath, "data")
        target_data_version_directory = os.path.join(target_data_directory, curr_version)
        bag_name = self.get_bag_name(identifier, curr_version, 1)
        target_data_version_asset_directory = os.path.join(target_data_version_directory, "content")
        tar_file_path = os.path.join(target_data_version_asset_directory, "%s.tar" % bag_name)
        if os.path.exists(tar_file_path):
            logger.debug("Package file found at: %s" % tar_file_path)
            return tar_file_path
        raise ObjectNotFoundException("Package file not found")

    def get_object_item_stream(self, identifier, entry, tar_file=None):
        """
        Get stream of a representation tar file entry
        :param identifier: package identifier
        :param entry: entry of the tar file
        :tar_file: path to tar file (optional)
        :return: chunks iterator of the tar file
        """
        tar_file_path = self.get_object_path(identifier)
        if os.path.exists(tar_file_path):
            logger.debug("tar file found at: %s" % entry)
        t = tar_file if tar_file else tarfile.open(tar_file_path, 'r')
        logger.debug("Accessing access entry %s" % entry)
        try:
            inst = ChunkedTarEntryReader(t)
            return inst.chunks(entry)
        except KeyError:
            logger.error('ERROR: Did not find %s in tar archive' % entry)
            raise ObjectNotFoundException("Entry not found in repository object")

    def trigger_new_version(self, uuid, identifier, config_path_work, storage_directory):
        """
        Trigger new version depending on changed files in working directory compared to the data set in storage.
        :param storage_directory:
        :param config_path_work:
        :param uuid: UUID of working directory
        :param identifier: Data asset identifier
        :return: True, if new version is triggered, False otherwise
        """
        working_dir = os.path.join(config_path_work, uuid)
        if self.identifier_object_exists(identifier):
            version = self.curr_version(identifier)
            data_asset_last_version_path = os.path.join(
                make_storage_data_directory_path(identifier, storage_directory),
                version, to_safe_filename(identifier))
            working_distributions_dir = os.path.join(working_dir, self.representations_directory)
            if not os.path.exists(working_distributions_dir):
                logger.debug("New version is not triggered because working catalogue directory does not exist.")
                return False
            stored_distributions_dir = os.path.join(data_asset_last_version_path, self.representations_directory)
            distribution_files = list_files_in_dir(working_distributions_dir)
            for dataset_dir in distribution_files:
                dataset_package_file = os.path.join(working_distributions_dir, "%s.tar" % dataset_dir)
                dataset_package_stored_file = os.path.join(stored_distributions_dir, "%s.tar" % dataset_dir)
                files_ident = files_identical(dataset_package_file, dataset_package_stored_file)
                if not files_ident:
                    logger.debug("New version triggered because hash of dataset packages is not identical")
                    return True
        logger.debug("New version not triggered.")
        return False


def update_state(state_xml_file, identifier, version):
    ip_state = IpState.from_path(state_xml_file) if os.path.exists(state_xml_file) \
        else IpState.from_parameters(0, False, last_task_value="stored")
    ip_state.set_version(version)
    ip_state.set_identifier(identifier)
    ip_state.write_doc(state_xml_file)


def get_package_from_directory_storage(task_context_path, package_uuid, package_extension,
                                       tl, config_path_storage):
    pts = PairtreeStorage(config_path_storage)
    parent_object_path = pts.get_object_path(package_uuid)

    package_in_dip_work_dir = os.path.join(task_context_path, ("%s.%s" % (package_uuid, package_extension)))
    package_source_size = fsize(parent_object_path)
    tl.addinfo("Source: %s (%d)" % (parent_object_path, package_source_size))
    tl.addinfo("Target: %s" % package_in_dip_work_dir)
    total_bytes_read = 0
    with open(package_in_dip_work_dir, 'wb') as target_file:
        for chunk in FileBinaryDataChunks(parent_object_path, 65536).chunks(total_bytes_read):
            target_file.write(chunk)
        total_bytes_read += package_source_size
        target_file.close()
        check_transfer(parent_object_path, package_in_dip_work_dir)


def make_storage_directory_path(identifier, version, config_path_storage):
    """Used for remote (no access to storage backend)"""
    pts = PairtreeStorage(config_path_storage)
    return os.path.join(pts.get_dir_path_from_id(identifier), "data", version, to_safe_filename(identifier))


def make_storage_data_directory_path(identifier, config_path_storage):
    """Used for remote (no access to storage backend)"""
    pts = PairtreeStorage(config_path_storage)
    return os.path.join(pts.get_dir_path_from_id(identifier), "data")


def get_package_from_storage(storage, task_context_path, package_uuid, package_extension, tl):

    pts = PairtreeStorage(storage)
    parent_object_path = pts.get_object_path(package_uuid)

    package_in_dip_work_dir = os.path.join(task_context_path, ("%s.%s" % (package_uuid, package_extension)))
    package_source_size = fsize(parent_object_path)
    tl.addinfo("Source: %s (%d)" % (parent_object_path, package_source_size))
    tl.addinfo("Target: %s" % package_in_dip_work_dir)
    total_bytes_read = 0
    with open(package_in_dip_work_dir, 'wb') as target_file:
        for chunk in FileBinaryDataChunks(parent_object_path, 65536).chunks(total_bytes_read):
            target_file.write(chunk)
        total_bytes_read += package_source_size
        target_file.close()
        check_transfer(parent_object_path, package_in_dip_work_dir)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import tarfile
import unittest

from eatb import ROOT
from eatb.packaging import ChunkedTarEntryReader


class ChunkedTarEntryReaderTest(unittest.TestCase):

    tar_test_file = None
    entry = None
    tfile = None

    @classmethod
    def setUpClass(cls):
        ChunkedTarEntryReaderTest.tar_test_file = os.path.join(ROOT, "tests/test_resources/storage-test/xyz.tar")
        ChunkedTarEntryReaderTest.entry = "xyz/representations/default/data/example.txt"
        ChunkedTarEntryReaderTest.tfile = tarfile.open(ChunkedTarEntryReaderTest.tar_test_file, 'r')

    def test_default_chunk_size(self):
        cter1 = ChunkedTarEntryReader(ChunkedTarEntryReaderTest.tfile)
        self.assertEqual(1, sum([1 for _ in cter1.chunks(ChunkedTarEntryReaderTest.entry)]))
        ChunkedTarEntryReader(ChunkedTarEntryReaderTest.tfile)

    def test_custom_chunk_size(self):
        cter2 = ChunkedTarEntryReader(ChunkedTarEntryReaderTest.tfile, 8192)
        self.assertEqual(1, sum([1 for _ in cter2.chunks(ChunkedTarEntryReaderTest.entry)]))
        ChunkedTarEntryReader(ChunkedTarEntryReaderTest.tfile)


if __name__ == '__main__':
    unittest.main()



from lxml import etree


class ParsedMets():
    """
    Parsed METS object
    """
    ns = {'mets': 'http://www.loc.gov/METS/', 'xlink': 'http://www.w3.org/1999/xlink',
          'xsi': 'http://www.w3.org/2001/XMLSchema-instance'}

    mets_tree = None

    def __init__(self, rdir):
        """
        Constructor takes root directory as argument; paths in METS file are relative to this directory.

        @type       rdir: string
        @param      rdir: Path to root directory
        """
        self.root_dir = rdir
        self.mets_tree = None

    def set_parsed_mets(self, pmets):
        """
        Set mets ElementTree object which is parsed already

        @type       pmets: ElementTree
        @param      pmets: Parsed METS
        """
        self.mets_tree = pmets

    def load_mets(self, mets_file_path):
        """
        Load mets Element tree object

        @type       pmets: ElementTree
        @param      pmets: Parsed METS
        """
        self.mets_tree = etree.parse(mets_file_path)

    def get_root(self):
        if self.mets_tree is None:
            raise ValueError("Attribute 'mets_tree' of type ElementTree must be initialized")
        return self.mets_tree.getroot()

    def get_file_elements(self):
        return self.mets_tree.getroot().xpath('/mets:mets/mets:fileSec/mets:fileGrp/mets:file', namespaces=ParsedMets.ns)

    def get_first_file_element(self):
        file_elements = self.get_file_elements()
        if len(file_elements) > 0:
            return file_elements[0]
        return None

    @staticmethod
    def get_file_element_checksum(file_element):
        return ''.join(file_element.xpath('@CHECKSUM'))

    @staticmethod
    def get_file_element_checksum_algorithm(file_element):
        return ''.join(file_element.xpath('@CHECKSUMTYPE'))

    @staticmethod
    def get_file_element_reference(file_element):
        return ''.join(file_element.xpath('mets:FLocat/@xlink:href', namespaces=ParsedMets.ns))

    def get_obj_id(self):
        xpath_result = self.mets_tree.getroot().xpath('@OBJID', namespaces=ParsedMets.ns)
        return xpath_result[0] if len(xpath_result) == 1 else "urn:uuid:none"

    def get_package_type(self):
        xpath_result = self.mets_tree.getroot().xpath('@TYPE', namespaces=ParsedMets.ns)
        return xpath_result[0] if len(xpath_result) == 1 else "NONE"

    def get_mets_schema_from_schema_location(self):
        if self.mets_tree is None:
            raise ValueError("Attribute 'mets_tree' of type ElementTree must be initialized")
        root = self.mets_tree.getroot()
        schema_file = ''
        locations = root.xpath('/mets:mets/@xsi:schemaLocation', namespaces=ParsedMets.ns)
        locations = ''.join(locations)
        locations = locations.split(' ')
        for token in locations:
            if token == ('http://www.loc.gov/METS/'):
                position = locations.index(token)
                schema_location = locations[position + 1]
                if schema_location.startswith(''):
                    schema_file = self.root_dir + schema_location
        return schema_file



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import unittest

import lxml.etree as ET

from eatb import ROOT
from eatb.oais_ip import DeliveryValidation
from eatb.utils.fileutils import read_file_content
from eatb.utils.xmlutils import get_dcat_element_values


class TestXml(unittest.TestCase):

    test_dir = "." + '/testresources/'
    delivery_dir = os.path.join(ROOT, 'tests/test_resources/Delivery-test/')

    schema_file = delivery_dir + 'schemas/mets.xsd'
    package_file = delivery_dir + 'pkg.tar.gz'
    vsip = DeliveryValidation()

    def test_get_dataset_property_value(self):
        """
        Test get dataset property value
        """
        md_file = os.path.join(ROOT + '/tests/test_resources/metadata/dcat/dcat2.xml')
        substitutions = {
            "//*/dcat:dataset/dcat:Dataset/dct:identifier": "xxx",
            "//*/dcat:dataset/dcat:Dataset/dcat:accessURL": "yyy",
            "//*/dcat:dataset/dcat:Dataset/dcat:distribution/dcat:Distribution/dcat:accessURL": "zzz"
        }
        xml_content = read_file_content(md_file)
        tree = ET.XML(xml_content.encode('utf-8'))
        distributions = set(tree.xpath("//*/dcat:dataset/dcat:Dataset/dcat:distribution/dcat:Distribution/dcat:accessURL", namespaces={'dct': 'http://purl.org/dc/terms/',
                                                                  'dcat': 'http://www.w3.org/ns/dcat#'}))

        changed_cont = get_dcat_element_values(md_file, substitutions)

        print(changed_cont)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

from lxml import objectify
from eatb.utils.XmlHelper import sequence_insert
from datetime import datetime
from lxml import etree, objectify
from eatb.utils.XmlHelper import q, XSI_NS
from eatb.utils.datetime import ts_date, DT_ISO_FORMAT_FILENAME

PREMIS_NS = 'http://www.loc.gov/premis/v3'
PREMIS_NSMAP = {None: PREMIS_NS}
P = objectify.ElementMaker(
    annotate=False,
    namespace=PREMIS_NS,
    nsmap=PREMIS_NSMAP)


class PremisCreator:

    premis_successor_sections = ['object', 'event', 'agent', 'right']
    working_dir = None

    def __init__(self, working_dir, f=None):
        self.working_dir = working_dir
        if f is None:
            self.root = P.premis(
                {q(XSI_NS, 'schemaLocation'): PREMIS_NS + ' https://www.loc.gov/standards/premis/v3/premis-v3-0.xsd'},
            )
            self.root.set('version', '3.0')
        else:
            self.root = objectify.parse(f).getroot()

    def add_object(self, identifier_value):
        sequence_insert(
            self.root, P.object(
                {q(XSI_NS, 'type'): 'file'},
                P.objectIdentifier(
                    P.objectIdentifierType('LOCAL'),
                    P.objectIdentifierValue(identifier_value)
                ),
            ),
            self.premis_successor_sections
        )

    def add_event(self, identifier_value, outcome, linking_agent, linking_object=None):
        sequence_insert(
            self.root, P.event(
                P.eventIdentifier(
                    P.eventIdentifierType('URN'),
                    P.eventIdentifierValue(identifier_value.format(ts_date(fmt=DT_ISO_FORMAT_FILENAME)))
                ),
                P.eventType,
                P.eventDateTime(datetime.utcnow().isoformat()),
                P.eventOutcomeInformation(
                    P.eventOutcome(outcome)
                ),
                P.linkingAgentIdentifier(
                    P.linkingAgentIdentifierType('URN'),
                    P.linkingAgentIdentifierValue(linking_agent)
                ),

                P.linkingAgentIdentifier(
                    P.linkingObjectIdentifierType('URN'),
                    P.linkingObjectIdentifierValue(linking_object)
                )
                if linking_object is not None else None
            ),
            self.premis_successor_sections
        )

    def add_relationship(self, identifier_value, outcome, linking_agent, linking_object=None):
        sequence_insert(
            self.root, P.event(
                P.eventIdentifier(
                    P.eventIdentifierType('URN'),
                    P.eventIdentifierValue(identifier_value.format(ts_date(fmt=DT_ISO_FORMAT_FILENAME)))
                ),
                P.eventType,
                P.eventDateTime(datetime.utcnow().isoformat()),
                P.eventOutcomeInformation(
                    P.eventOutcome(outcome)
                ),
                P.linkingAgentIdentifier(
                    P.linkingAgentIdentifierType('URN'),
                    P.linkingAgentIdentifierValue(linking_agent)
                ),

                P.linkingAgentIdentifier(
                    P.linkingObjectIdentifierType('URN'),
                    P.linkingObjectIdentifierValue(linking_object)
                )
                if linking_object is not None else None
            ),
            self.premis_successor_sections
        )

    def add_agent(self, identifier_value, agent_name, agent_type):
        sequence_insert(
            self.root, P.agent(
                P.agentIdentifier(
                    P.agentIdentifierType('URN'),
                    P.agentIdentifierValue(identifier_value)
                ),
                P.agentName(agent_name),
                P.agentType(agent_type)
            ),
            self.premis_successor_sections
        )

    def __str__(self):
        return etree.tostring(self.root, encoding='UTF-8', pretty_print=True, xml_declaration=True)

    def to_string(self):
        return etree.tostring(self.root, encoding='UTF-8', pretty_print=True, xml_declaration=True)

    def create(self, path, type, subtype):
        premis_file_name = "premis_earkweb_%s_%s_%s.xml" % (ts_date(fmt=DT_ISO_FORMAT_FILENAME), type, subtype)
        path_premis = os.path.join(self.working_dir, path, premis_file_name)
        with open(path_premis, 'w') as output_file:
            output_file.write(self.to_string().decode('UTF-8'))





import hashlib
import os
import uuid
from mimetypes import MimeTypes
from subprocess import PIPE, Popen

from lxml import etree

from eatb.file_format import FormatIdentification
from eatb.metadata.parsed_premis import P
from eatb.settings import fido_enabled
from eatb.utils.XmlHelper import q, XSI_NS
from eatb.utils.datetime import current_timestamp


class PremisGenerator(object):
    fid = FormatIdentification()
    mime = MimeTypes()
    root_path = ""

    def __init__(self, root_path):
        self.root_path = root_path

    def sha256(self, fname):
        hash = hashlib.sha256()
        with open(fname, 'rb') as f:
            for chunk in iter(lambda: f.read(4096), b""):
                hash.update(chunk)
        return hash.hexdigest()

    def runCommand(self, program, stdin = PIPE, stdout = PIPE, stderr = PIPE):
        result, res_stdout, res_stderr = None, None, None
        try:
            # quote the executable otherwise we run into troubles
            # when the path contains spaces and additional arguments
            # are presented as well.
            # special: invoking bash as login shell here with
            # an unquoted command does not execute /etc/profile

            print('Launching: ' + ' '.join(program))
            process = Popen( program, stdin = stdin, stdout = stdout, stderr = stderr, shell = False)

            res_stdout, res_stderr = process.communicate()
            result = process.returncode
            print('Finished: ' + ' '.join(program))

        except Exception as ex:
            res_stderr = ''.join(str(ex.args))
            result = 1

        if result != 0:
            print('Command failed:' + ''.join(res_stderr))
            raise Exception('Command failed:' + ''.join(res_stderr))

        return result, res_stdout, res_stderr

    def addObject(self, abs_path):
        '''
        Must be called with the absolute path to a file.

        @param abs_path:    absolute file path
        @return:            Premis object
        '''

        hash = self.sha256(abs_path)
        file_url = "%s" % os.path.relpath(abs_path, self.root_path)
        fmt = None
        if fido_enabled:
            fmt = self.fid.identify_file(abs_path)
        size = os.path.getsize(abs_path)
        premis_id = 'ID' + uuid.uuid4().__str__()

        if not fmt:
            fmt = 'fmt/000'

        # create a Premis object
        object = P.object(
            {q(XSI_NS, 'type'): 'file', "xmlID": premis_id},
            P.objectIdentifier(
                P.objectIdentifierType('filepath'),
                P.objectIdentifierValue(file_url)
            ),
            P.objectCharacteristics(
                P.compositionLevel(0),
                P.fixity(
                    P.messageDigestAlgorithm("SHA-256"),
                    P.messageDigest(hash),
                    P.messageDigestOriginator("hashlib")
                ),
                P.size(size),
                P.format(
                    P.formatRegistry(
                        P.formatRegistryName("PRONOM"),
                        P.formatRegistryKey(fmt),
                        P.formatRegistryRole("identification")
                    )
                ),
            ),
        )
        return object

    def addEvent(self, premispath, info):
        '''
        Add an event to an exisiting Premis file (DefaultTask finalize method).

        @param premispath:
        @param info:
        @return:
        '''
        # print type(premispath)

        outcome = info['outcome']
        agent = info['task_name']
        event_type = info['event_type']
        linked_object = info['linked_object']

        premis_path = os.path.join(self.root_path, premispath)
        premis_parsed = etree.parse(premis_path)
        premis_root = premis_parsed.getroot()

        event_id = 'ID' + uuid.uuid4().__str__()
        event = P.event(
            P.eventIdentifier(
                P.eventIdentifierType('local'),
                P.eventIdentifierValue(event_id)),
            P.eventType(event_type),
            P.eventDateTime(current_timestamp()),
            P.eventOutcomeInformation(
                P.eventOutcome(outcome)
            ),
            P.linkingAgentIdentifier(
                P.linkingAgentIdentifierType('software'),
                P.linkingAgentIdentifierValue('Premis Generator')),
            P.linkingObjectIdentifier(
                P.linkingObjectIdentifierType('repository'),
                P.linkingObjectIdentifierValue(linked_object))
        )
        premis_root.insert(len(premis_root) - 1, event)

        str = etree.tostring(premis_root, encoding='UTF-8', pretty_print=True, xml_declaration=True)
        with open(premis_path, 'w') as output_file:
            output_file.write(str.decode('utf-8'))

        return

    def createMigrationPremis(self, premis_info):
        PREMIS_ATTRIBUTES = {"version" : "2.0"}
        premis = P.premis(PREMIS_ATTRIBUTES)
        premis.attrib['{%s}schemaLocation' % XSI_NS] = "info:lc/xmlns/premis-v2 ../../schemas/premis-v2-2.xsd"

        # creates an object that references the package or representation
        # TODO: identifier!
        premis_id = 'ID' + uuid.uuid4().__str__()
        object = P.object(
            {q(XSI_NS, 'type'): 'representation', "xmlID": premis_id},
            P.objectIdentifier(
                P.objectIdentifierType('repository'),
                P.objectIdentifierValue(premis_id)
            ),
        )
        premis.append(object)

        # list of software agents
        software_agents = []

        # parse the migration.xml, add events and objects
        with open(premis_info['info'], mode='rb') as premis_info_entity:
            migrations = etree.iterparse(premis_info_entity, events=('start',))
            eventlist = []
            for event, element in migrations:
                if element.tag == 'migration':
                    event_id = 'ID' + uuid.uuid4().__str__()
                    if self.root_path.endswith(element.attrib['targetrep']):
                        source_object_abs = os.path.join(element.attrib['sourcedir'], element.attrib['file'])
                        source_object_rel = "%s" % os.path.relpath(source_object_abs, self.root_path)
                        target_object_abs = os.path.join(element.attrib['targetdir'], element.attrib['output'])
                        target_object_rel = "%s" % os.path.relpath(target_object_abs, self.root_path)

                        # add agent to list
                        if element.attrib['agent'] not in software_agents:
                            software_agents.append(element.attrib['agent'])

                        # event
                        event = P.event(
                            P.eventIdentifier(
                                P.eventIdentifierType('local'),
                                P.eventIdentifierValue(event_id)),
                            P.eventType('migration'),
                            P.eventDateTime(element.attrib['starttime']),
                            P.eventOutcomeInformation(
                                P.eventOutcome('success')
                            ),
                            P.linkingAgentIdentifier(
                                P.linkingAgentIdentifierType('software'),
                                P.linkingAgentIdentifierValue(element.attrib['agent'])),
                            P.linkingObjectIdentifier(
                                P.linkingObjectIdentifierType('filepath'),
                                P.linkingObjectIdentifierValue(target_object_rel))
                        )
                        eventlist.append(event)

                        # object
                        object = self.addObject(target_object_abs)
                        # add the relationship to the migration event and the source file
                        relationship = P.relationship(
                            P.relationshipType('derivation'),
                            P.relationshipSubType('has source'),
                            P.relatedObjectIdentification(
                                P.relatedObjectIdentifierType('filepath'),
                                P.relatedObjectIdentifierValue(source_object_rel),
                                P.relatedObjectSequence('0')
                            ),
                            P.relatedEventIdentification(
                                P.relatedEventIdentifierType('local'),
                                P.relatedEventIdentifierValue(event_id),
                                P.relatedEventSequence('1')
                            ),
                        )
                        object.append(relationship)

                        premis.append(object)
                    else:
                        pass
                else:
                    pass

            # append all events to premis root - they must be below the objects (due to validation)
            for event in eventlist:
                premis.append(event)

            # add conduit agent
            identifier_value = 'Premis Generator'
            premis.append(P.agent(
                    P.agentIdentifier(
                        P.agentIdentifierType('LOCAL'),
                        P.agentIdentifierValue(identifier_value)
                    ),
                    P.agentName('Premis Generator'),
                    P.agentType('Software')))

            # add software agents
            for agent in software_agents:
                premis.append(P.agent(
                        P.agentIdentifier(
                            P.agentIdentifierType('LOCAL'),
                            P.agentIdentifierValue(agent)
                        ),
                        P.agentName(agent),
                        P.agentType('Software')))

            # create the Premis file
            str = etree.tostring(premis, encoding='UTF-8', pretty_print=True, xml_declaration=True)
            preservation_dir = os.path.join(self.root_path, 'metadata/preservation')
            if not os.path.exists(preservation_dir):
                os.makedirs(preservation_dir)
            path_premis = os.path.join(self.root_path, 'metadata/preservation/premis.xml')
            with open(path_premis, 'w') as output_file:
                output_file.write(str.decode('utf-8'))

            return

    def createPremis(self, path_premis=None):
        PREMIS_ATTRIBUTES = {"version" : "3.0"}
        premis = P.premis(PREMIS_ATTRIBUTES)
        premis.attrib['{%s}schemaLocation' % XSI_NS] = "http://www.loc.gov/premis/v3 https://www.loc.gov/standards/premis/v3/premis-v3-0.xsd"

        # if there are no /data files, this will ensure that there is at least one object (the IP itself)
        premis_id = 'ID' + uuid.uuid4().__str__()
        object = P.object(
            {q(XSI_NS, 'type'): 'representation', "xmlID": premis_id},
            P.objectIdentifier(
                P.objectIdentifierType('repository'),
                P.objectIdentifierValue(premis_id)
            ),
        )
        premis.append(object)

        # create premis objects for files in this representation (self.root_path/data)
        for directory, subdirectories, filenames in os.walk(os.path.join(self.root_path, 'data')):
            for filename in filenames:
                object = self.addObject(os.path.join(directory, filename))
                premis.append(object)

        # # event
        # identifier_value = 'AIP Creation'
        # linking_agent = 'conduit'
        # linking_object=None
        # premis.append(P.event(
        #         P.eventIdentifier(
        #             P.eventIdentifierType('local'),
        #             P.eventIdentifierValue(identifier_value)
        #         ),
        #         P.eventType,
        #         P.eventDateTime(current_timestamp()),
        #         P.linkingAgentIdentifier(
        #             P.linkingAgentIdentifierType('local'),
        #             P.linkingAgentIdentifierValue(linking_agent)
        #         ),
        #
        #         P.linkingAgentIdentifier(
        #             P.linkingAgentIdentifierType('local'),
        #             P.linkingAgentIdentifierValue(linking_object)
        #         )
        #         if linking_object is not None else None
        #     ))

        # add agent
        identifier_value = 'Premis Generator'
        premis.append(P.agent(
            P.agentIdentifier(
                P.agentIdentifierType('LOCAL'),
                P.agentIdentifierValue(identifier_value)
            ),
            P.agentName('Premis Generator'),
            P.agentType('Software')))

        str = etree.tostring(premis, encoding='UTF-8', pretty_print=True, xml_declaration=True)
        preservation_dir = os.path.join(self.root_path, 'metadata/preservation')
        if not os.path.exists(preservation_dir):
            os.makedirs(preservation_dir)

        if not path_premis:
            path_premis = os.path.join(self.root_path, 'metadata/preservation/premis.xml')

        os.makedirs(os.path.dirname(path_premis), exist_ok=True)

        with open(path_premis, 'w') as output_file:
            output_file.write(str.decode('utf-8'))

        return



import os
import unittest
import tarfile
import sys
from eatb.utils.randomutils import randomword
from eatb import ROOT

IP_path = os.path.join(ROOT, 'tests/test_resources/minimal_IP_with_schemas')
rules_path = os.path.join(ROOT, 'tests/test_resources/rules.xml')


class Base(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        self.IP_path = IP_path
        with open(rules_path) as fp:
            xml_rules = fp.read()
        self.rules_lines = xml_rules.split('\n')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json

try:
    from fido.fido import Fido
    fido_disabled = False
except ImportError:
    fido_disabled = True


class FormatIdentification:
    """
    File Format Identification
    """

    def __init__(self, format_files=None):
        fmt_files = format_files
        if not format_files:
            from eatb.settings import format_files_cfg
            fmt_files = format_files_cfg.split(",")
        if not fido_disabled:
            self.fid = Fido(format_files=fmt_files)
            self.fid.handle_matches = self.print_matches
            self.lastFmt = None

    def identify_file(self, entry):
        """
        This function identifies the file format of every file that is handed over.
        """
        assert not fido_disabled, "Fido module is not available!"
        self.fid.identify_file(entry)
        return self.lastFmt

    def get_mime_for_puid(self, puid):
        """

        :param puid: PRONOM Persistent Unique Identifier
        :return: mime type string (default: application/octet-stream)
        """
        assert not fido_disabled, "Fido module is not available!"
        mime_tag = "mime"
        fmtres = self.fid.puid_format_map[puid]
        childs = [child for child in fmtres if child.tag.endswith(mime_tag)]
        if len(childs) == 1:
            return (childs[0]).text.strip()
        return "application/octet-stream"

    def print_matches(self, fullname, matches, delta_t, matchtype=''):
        assert not fido_disabled, "Fido module is not available!"
        for (f, s) in matches:
            self.lastFmt = self.fid.get_puid(f)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import string
import unittest

from eatb.cli import CliCommand, CliCommands
from eatb.cli import CliExecution


class CliCommandTest(unittest.TestCase):

    def test_get_cli_command(self):
        cli_command = CliCommand("echo", ['echo', string.Template('$input')])
        command = cli_command.get_command({'input': 'foo'})
        cli_execution = CliExecution(command)
        process_output = cli_execution.execute()
        print(process_output)

    def test_cli_commands(self):
        cli_commands = CliCommands()
        cli_command = CliCommand("echo", cli_commands.get_command_template("echo"))
        command = cli_command.get_command({'input': "test"})
        self.assertIsNotNone(command)
        self.assertEqual(2, len(command))
        self.assertEqual('echo', command[0])
        self.assertEqual('test', command[1])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil
import unittest

from eatb import ROOT
from eatb.metadata.parsed_mets import ParsedMets
from eatb.utils import randomutils
from eatb.metadata.mets_generator import MetsGenerator


class TestMetsCreation(unittest.TestCase):

    test_ip = "eark-ip"
    test_source_directory = os.path.join(ROOT + '/tests/test_resources/')
    temp_working_dir = os.path.join(ROOT, "tmp/tmp-%s" % randomutils.randomword(10))
    tmp_ip_dir = os.path.join(temp_working_dir, test_ip)

    test_ip_dir = os.path.join(test_source_directory, test_ip)

    mets_file_path = os.path.join(temp_working_dir, test_ip, 'METS.xml')

    @classmethod
    def setUpClass(cls):
        shutil.copytree(TestMetsCreation.test_ip_dir, TestMetsCreation.tmp_ip_dir)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(TestMetsCreation.temp_working_dir)
        pass

    @unittest.skip("Currently throws a KeyError:'data' error @{}". \
        format("eatb/metadata/mets/mets.py:446"))
    def testCreateMets(self):
        metsgen = MetsGenerator(TestMetsCreation.test_ip_dir)
        mets_data = {'packageid': '996ed635-3e13-4ee5-8e5b-e9661e1d9a93',
                     'type': 'AIP', 'schemas': 'schemas', 'parent': None}
        metsgen.createMets(mets_data, TestMetsCreation.mets_file_path)
        self.assertTrue(os.path.exists(TestMetsCreation.mets_file_path))

        pm = ParsedMets(TestMetsCreation.test_ip_dir)
        pm.load_mets(TestMetsCreation.mets_file_path)

        obj_id = pm.get_obj_id()
        self.assertEqual("996ed635-3e13-4ee5-8e5b-e9661e1d9a93", obj_id)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import unittest

from eatb import ROOT
from eatb.csip_validation import CSIPValidation

class TestCSIPValidation(unittest.TestCase):

    def test_csip_validation(self):
        ip_path = os.path.join(ROOT, 'tests/test_resources/metadata/mets/')
        rules_path = os.path.join(ROOT, "tests/test_resources/rules.xml")
        csip_validation = CSIPValidation(rules_path)
        validation_report = csip_validation.validate(ip_path)
        self.assertTrue(len(validation_report) > 0, str(len(validation_report)))
        self.assertTrue(_has_part_key('RootElement', validation_report), str(validation_report))
        self.assertTrue(len(_get_part('RootElement', validation_report)) > 0)
        self.assertTrue(_get_part('RootElement', validation_report)[0]['rule'] == "csip1")
        self.assertTrue(_get_part('RootElement', validation_report)[0]['result'])

    def test_csip_validation_with_default_rules(self):
        ip_path = os.path.join(ROOT, 'tests/test_resources/metadata/mets/')
        csip_validation = CSIPValidation()
        validation_report = csip_validation.validate(ip_path)
        self.assertTrue(len(validation_report) > 0 )
        self.assertTrue(len(validation_report) > 0)
        self.assertTrue(str(csip_validation.get_rules_file_path()).endswith("eatb/eatb/resources/validation_rules.xml"))

    def test_get_log_lines(self):
        ip_path = os.path.join(ROOT, 'tests/test_resources/metadata/mets/')
        csip_validation = CSIPValidation()
        csip_validation.validate(ip_path)
        self.assertTrue(len(csip_validation.get_log_lines()) > 0)

def _has_part_key(part_key, report):
    for report_part in report:
        if part_key in report_part:
            return True
    return False

def _get_part(part_key, report):
    for report_part in report:
        if part_key in report_part:
            return report_part[part_key]
    return None

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil
import unittest

from eatb import ROOT
from eatb.packaging import ZipContainer
from eatb.utils.randomutils import randomword


class TestExtraction(unittest.TestCase):

    test_resources_dir = os.path.join(ROOT, 'tests/test_resources')
    temp_extract_dir = '/tmp/backend-unzip-' + randomword(10)

    @classmethod
    def setUpClass(cls):
        os.makedirs(TestExtraction.temp_extract_dir)
        zip_container_test_resource_path = os.path.join(TestExtraction.test_resources_dir, "package.zip")
        shutil.copy(zip_container_test_resource_path, TestExtraction.temp_extract_dir)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(TestExtraction.temp_extract_dir)
        pass

    def test_extract_sip(self):
        print("Extracting to %s" % self.temp_extract_dir)
        zip_container_file_path = os.path.join(TestExtraction.temp_extract_dir, "package.zip")
        container_folder = "package"
        zip_container = ZipContainer(zip_container_file_path)
        success = zip_container.unzip(TestExtraction.temp_extract_dir)
        self.assertTrue(success)
        cpt = sum([len(files) for r, d, files in os.walk(TestExtraction.temp_extract_dir)])
        self.assertEqual(cpt, 3, "Number of extracted files not as expected")
        files_to_check = (
            os.path.join(TestExtraction.temp_extract_dir, container_folder, './first_level.txt'),
            os.path.join(TestExtraction.temp_extract_dir, container_folder, './subfolder/second_level.txt'),
        )
        for file in files_to_check:
            self.assertTrue(os.path.isfile(file), "File %s not found in extracted directory" + file)


if __name__ == '__main__':
    unittest.main()



import logging
import os
from enum import Enum
from pathlib import Path
import uuid

import lxml.etree as ET

from lxml import objectify

from eatb import ROOT
from eatb.checksum import ChecksumValidation, ChecksumAlgorithm
from eatb.metadata.mets_generator import MetsGenerator
from eatb.metadata.parsed_mets import ParsedMets

from eatb.metadata.premis_generator import PremisGenerator
from eatb.packaging import create_package
from eatb.csip_validation import ValidationResult, XmlValidation
from eatb.file_format import FormatIdentification

import hashlib
from mimetypes import MimeTypes

from subprocess import Popen, PIPE

from eatb.utils.XmlHelper import XSI_NS, q
from eatb.utils.datetime import get_file_ctime_iso_date_str, DT_ISO_FMT_SEC_PREC, current_timestamp


LOGGER = logging.getLogger(__name__)


class OAISPackageType(Enum):
    SIP = 1
    AIP = 2
    DIP = 3
    IP = 4


def create_sip(package_dir: str, package_name: str, identifier: str, generate_premis: bool=True,
               generate_package: bool=True, custom_logger=None) -> bool:

    logger = custom_logger if custom_logger else LOGGER

    # PREMIS
    premis_path = os.path.join(package_dir, 'metadata/preservation/premis.xml')
    if generate_premis:
        premisgen = PremisGenerator(package_dir)

        premisgen.createPremis()
        premisinfo = {'outcome': 'success',
                      'task_name': 'create_sip',
                      'event_type': 'Package initialisation',
                      'linked_object': None}

        logger.info("Creating PREMIS file %s" % premis_path)
        premisgen.addEvent('metadata/preservation/premis.xml', premisinfo)

    reps_path = os.path.join(package_dir, 'representations')
    for name in os.listdir(reps_path):
        rep_path = os.path.join(reps_path, name)
        if os.path.isdir(rep_path):
            if generate_premis:
                # representation PREMIS
                logger.info(
                    "Creating representation PREMIS file %s" %
                    os.path.join(rep_path, 'metadata/preservation/premis.xml'))
                premisgen = PremisGenerator(rep_path)
                premisgen.createPremis()
            # representation METS
            logger.info("Creating representation METS file: %s" % os.path.join(rep_path, "METS.xml"))
            mets_data = {'packageid': package_name,
                         'type': "SIP",
                         'schemas': os.path.join(package_dir, 'schemas'),
                         'parent': ''}
            metsgen = MetsGenerator(rep_path)

            metsgen.createMets(mets_data)
    if generate_premis:
        # PREMIS
        premisgen = PremisGenerator(package_dir)
        premisgen.createPremis()

    # METS
    mets_data = {'packageid': package_name,
                 'type': "SIP",
                 'schemas': os.path.join(package_dir, 'schemas'),
                 'parent': ''}
    mets_path = os.path.join(package_dir, "METS.xml")
    logger.info("Creating METS file %s" % mets_path)
    metsgen = MetsGenerator(package_dir)
    metsgen.createMets(mets_data)

    # packaging
    input_path = Path(package_dir)
    storage_tar_file = os.path.join(input_path.parent, package_name + '.tar')
    if generate_package:
        logger.info("Creating package: %s" % storage_tar_file)
        create_package(package_dir, package_name)

    if not os.path.exists(mets_path):
        logger.error("Error creating METS file %s" % mets_path)
    premis_success = (not generate_package or os.path.exists(premis_path))
    if not premis_success:
        logger.error("Error creating PREMIS file %s" % premis_path)
    package_success = (not generate_package or os.path.exists(storage_tar_file))
    if not package_success:
        logger.error("Error creating package %s" % storage_tar_file)

    return (
        os.path.exists(mets_path)
        and
        premis_success
        and
        package_success
    )



logger = logging.getLogger(__name__)


def create_aip(package_dir, identifier, package_name, identifier_map=None, generate_premis=True,
               generate_package=True) -> bool:

    # schema file location for Mets generation
    schemas = os.path.join(ROOT, 'resources/schemas')

    # for every REPRESENTATION without METS file:
    for repdir in os.listdir(os.path.join(package_dir, 'representations')):
        rep_path = os.path.join(package_dir, 'representations/%s' % repdir)
        # TODO: packageid?
        # parent = task_context.additional_data['parent_id']
        mets_data = {'packageid': repdir,
                     'type': "AIP",
                     'schemas': schemas,
                     'parent': ''}
        metsgen = MetsGenerator(rep_path)
        metsgen.createMets(mets_data)

        logger.info('Generated a Mets file for representation %s.' % repdir)

    #########

    # schema file location for Mets generation
    schemas = os.path.join(package_dir, 'schemas')

    mets_data = {'packageid': identifier,
                 'type': "AIP",
                 'schemas': schemas,
                 'parent': None}

    metsgen = MetsGenerator(package_dir)
    metsgen.createMets(mets_data)

    # TODO: get structMap from submission root METS, translate SIP packagename to
    # AIP UUID and add it to the AIP root METS

    # get identifier_map
    if identifier_map:
        # check if packagename is in the identifier map
        if package_name in identifier_map:
            identifier = identifier_map[package_name]
            logger.info("Provided identifier assigned: %s" % identifier)
        else:
            logger.error("Cannot find uuid for package name %s in provided identifier_map" % package_name)


        METS_NS = 'http://www.loc.gov/METS/'
        XLINK_NS = "http://www.w3.org/1999/xlink"

        # parse submission SIP mets
        parser = ET.XMLParser(resolve_entities=False, remove_blank_text=True, strip_cdata=False)
        sip_mets_path = os.path.join(package_dir, 'submission', 'METS.xml')
        # print sip_mets_path
        sip_parse = ET.parse(sip_mets_path, parser)
        sip_root = sip_parse.getroot()

        # parse package AIP METS and append children and parent structMaps
        aip_mets_path = os.path.join(package_dir, 'METS.xml')
        aip_parse = ET.parse(aip_mets_path, parser)
        aip_root = aip_parse.getroot()

        # get children structMap and replace the urn:uuid:packagename with corresponding urn:uuid:uuid
        children_map = sip_root.find("%s[@LABEL='child %s']" % (q(METS_NS, 'structMap'), 'SIP'))
        if children_map is not None:
            children_map.set('LABEL', 'child AIP')
            children_div = children_map.find("%s[@LABEL='child %s identifiers']" % (q(METS_NS, 'div'), 'SIP'))
            children_div.set('LABEL', 'child AIP identifiers')
            children = children_div.findall("%s[@LABEL='child %s']" % (q(METS_NS, 'div'), 'SIP'))
            for child in children:
                child.set('LABEL', 'child AIP')
                mptr = child.find("%s" % q(METS_NS, 'mptr'))
                urn = mptr.get(q(XLINK_NS, 'href'))
                urn = urn.split('urn:uuid:', 1)[1]
                uuid = identifier_map[urn]
                mptr.set(q(XLINK_NS, 'href'), 'urn:uuid:' + uuid)
                mptr.set(q(XLINK_NS, 'title'), 'Referencing a child AIP.')
            aip_root.append(children_map)
        # get parents structMap and replace the urn:uuid:packagename with corresponding urn:uuid:uuid
        parents_map = sip_root.find("%s[@LABEL='parent %s']" % (q(METS_NS, 'structMap'), 'SIP'))
        if parents_map is not None:
            parents_map.set('LABEL', 'parent AIP')
            parents_div = parents_map.find("%s[@LABEL='parent %s identifiers']" % (q(METS_NS, 'div'), 'SIP'))
            parents_div.set('LABEL', 'parent AIP identifiers')
            parents = parents_div.findall("%s[@LABEL='parent %s']" % (q(METS_NS, 'div'), 'SIP'))
            for parent in parents:
                parent.set('LABEL', 'parent AIP')
                mptr = parent.find("%s" % q(METS_NS, 'mptr'))
                urn = mptr.get(q(XLINK_NS, 'href'))
                urn = urn.split('urn:uuid:', 1)[1]
                uuid = identifier_map[urn]
                mptr.set(q(XLINK_NS, 'href'), 'urn:uuid:' + uuid)
                mptr.set(q(XLINK_NS, 'title'), 'Referencing a parent AIP.')
            aip_root.append(parents_map)

        str = ET.tostring(aip_root, encoding='UTF-8', pretty_print=True, xml_declaration=True)
        with open(aip_mets_path, 'w') as output_file:
            output_file.write(str)

    # packaging
    input_path = Path(package_dir)
    storage_tar_file = os.path.join(input_path.parent, identifier + '.tar')
    if generate_package:
        logger.info("Creating package: %s" % storage_tar_file)
        create_package(package_dir, identifier)

    mets_path = os.path.join(package_dir, "METS.xml")
    if not os.path.exists(mets_path):
        logger.error("Error creating METS file %s" % mets_path)
    else:
        logger.info('Generated a root Mets file: %s.' % mets_path)

    package_success = (not generate_package or os.path.exists(storage_tar_file))
    if not package_success:
        logger.error("Error creating package %s" % storage_tar_file)

    return (
        os.path.exists(mets_path)
        and
        package_success
    )


class DeliveryValidation(object):

    def getFileElements(self, deliveryDir, delivery_xml_file, schema_file):
        log = []
        log.append("Validating delivery: %s using schema: %s" % (delivery_xml_file, schema_file))
        try:
            # Parse the XML file, get the root element
            parsed_mets = ParsedMets(deliveryDir)
            parsed_mets.load_mets(delivery_xml_file)
            # If the XSD file wasn't found, extract location from the XML
            if schema_file == None:
                schema_file = parsed_mets.get_mets_schema_from_schema_location()
            # Parse the XSD file
            parsed_sfile = ET.parse(schema_file)
            # Validate the delivery XML file
            xmlVal = XmlValidation()
            validation_result = xmlVal.validate_XML(parsed_mets.mets_tree, parsed_sfile)
            if validation_result:
                return parsed_mets.get_file_elements()
        except ET.XMLSyntaxError as why:
            logger.error('Error validating delivery %s, why: %s' % (delivery_xml_file, str(why)))
        return None

    def validate_delivery(self, deliveryDir, checksum_file, schema_file, package_file):
        """
        Validate the delivery document.
        level.

        @type       deliveryDir: string
        @param      deliveryDir: Path to delivery directory
        @type       delivery_xml_file:  string
        @param      delivery_xml_file:  Path to delivery file.
        @type       package_file:  string
        @param      package_file:  Path to package file (e.g. TAR).
        @rtype:     ValidationResult
        @return:    Validation result (validity, process log, error log)
        """
        valid = False
        log = []
        err = []
        valid_checksum = False
        # TODO: define
        checksum_algorithm = ""
        # Checksum validation
        self.get_checksum()
        checksum_expected = self.get_checksum(deliveryDir, checksum_algorithm)
        csval = ChecksumValidation()
        valid_checksum = csval.validate_checksum(package_file, checksum_expected, ChecksumAlgorithm.get(checksum_algorithm))
        log.append("Checksum validity: \"%s\"" % str(valid_checksum))
        valid = (valid_checksum)
        return ValidationResult(valid, log, err)


METS_NS = 'http://www.loc.gov/METS/'
METSEXT_NS = 'ExtensionMETS'
PREMIS_NS = 'http://www.loc.gov/premis/v3'
PREMIS_NSMAP = {None: PREMIS_NS}
XLINK_NS = "http://www.w3.org/1999/xlink"
METS_NSMAP = {None: METS_NS, "xlink": "http://www.w3.org/1999/xlink", "ext": METSEXT_NS,
                  "xsi": "http://www.w3.org/2001/XMLSchema-instance"}
DELIVERY_METS_NSMAP = {None: METS_NS, "xlink": "http://www.w3.org/1999/xlink",
                           "xsi": "http://www.w3.org/2001/XMLSchema-instance"}
P = objectify.ElementMaker(
        annotate=False,
        namespace=PREMIS_NS,
        nsmap=PREMIS_NSMAP)
M = objectify.ElementMaker(
    annotate=False,
    namespace=METS_NS,
    nsmap=METS_NSMAP)


class SIPGenerator():
    fid = FormatIdentification()
    mime = MimeTypes()
    root_path = ""

    def __init__(self, root_path):
        self.root_path = root_path

    def sha256(self, fname):

        sha256_hash = hashlib.sha256()
        with open(fname, "rb") as f:
            # Read and update hash string value in blocks of 4K
            for byte_block in iter(lambda: f.read(4096), b""):
                sha256_hash.update(byte_block)
            return sha256_hash.hexdigest()

    def createAgent(self,role, type, other_type, name, note):
        if other_type:
            agent = M.agent({"ROLE":role,"TYPE":type, "OTHERTYPE": other_type}, M.name(name), M.note(note))
        else:
            agent = M.agent({"ROLE":role,"TYPE":type}, M.name(name), M.note(note))
        return agent

    def runCommand(self, program, stdin = PIPE, stdout = PIPE, stderr = PIPE):
        result, res_stdout, res_stderr = None, None, None
        try:
            # quote the executable otherwise we run into troubles
            # when the path contains spaces and additional arguments
            # are presented as well.
            # special: invoking bash as login shell here with
            # an unquoted command does not execute /etc/profile

            print('Launching: ' + ' '.join(program))
            process = Popen( program, stdin = stdin, stdout = stdout, stderr = stderr, shell = False)

            res_stdout, res_stderr = process.communicate()
            result = process.returncode
            print('Finished: ' + ' '.join(program))

        except Exception as ex:
            res_stderr = ''.join(str(ex.args))
            result = 1

        if result != 0:
            print('Command failed:' + ''.join(res_stderr))
            raise Exception('Command failed:' + ''.join(res_stderr))

        return result, res_stdout, res_stderr

    def addFile(self, file_name, mets_filegroup):
        #reload(sys)
        #sys.setdefaultencoding('utf8')
        file_url = "%s" % os.path.relpath(file_name, self.root_path)
        file_mimetype,_ = self.mime.guess_type(file_url)
        file_mimetype = file_mimetype if file_mimetype else "application/octet-stream"
        file_checksum = self.sha256(file_name)
        file_size = os.path.getsize(file_name)
        file_cdate = get_file_ctime_iso_date_str(file_name, DT_ISO_FMT_SEC_PREC)
        file_id = "ID"+uuid.uuid4().__str__()
        mets_file = M.file({"MIMETYPE":file_mimetype, "CHECKSUMTYPE":"SHA-256", "CREATED":file_cdate, "CHECKSUM":file_checksum, "USE":"Datafile", "ID":file_id, "SIZE":file_size})
        mets_filegroup.append(mets_file)
        #_,fname = os.path.split(file_name)
        mets_FLocat = M.FLocat({q(XLINK_NS, 'href'): file_url, "LOCTYPE":"URL", q(XLINK_NS, 'type'): 'simple'})
        mets_file.append(mets_FLocat)
        return file_id

    def addFiles(self, folder, mets_filegroup):
        ids = []
        for top, dirs, files in os.walk(folder):
            for fn in files:
                file_name = os.path.join(top, fn)
                file_id = self.addFile(file_name,mets_filegroup)
                ids.append(file_id)
        return ids

    # def createPremis(self, enable_jhove = False):
    #     jhove_parser = None
    #     if enable_jhove == True:
    #         jhove_parser = etree.XMLParser(remove_blank_text=True)
    #
    #     PREMIS_ATTRIBUTES = {"version" : "2.0"}
    #     premis = P.premis(PREMIS_ATTRIBUTES)
    #     premis.attrib['{%s}schemaLocation' % XSI_NS] = "http://www.loc.gov/premis/v3 ../../schemas/premis-v3-0.xsd"
    #
    #     premis_ids = []
    #     for top, dirs, files in os.walk(os.path.join(self.root_path, 'data')):
    #         for nm in files:
    #             file_name = os.path.join(top,nm)
    #             hash = self.sha256(file_name)
    #             file_url = "./%s" % os.path.relpath(file_name, self.root_path)
    #             fmt = self.fid.identify_file(file_name)#os.path.abspath(remove_protocol(file_url)))
    #             jhove = None
    #             if enable_jhove == True:
    #                 try:
    #                     result = self.runCommand(["/usr/bin/jhove", "-h", "xml", file_name] )
    #                     if result[0] == 0:
    #                         jhove = etree.XML(result[1], parser=jhove_parser)
    #                 except Exception:
    #                     #TODO: handle exception
    #                     pass
    #
    #             size = os.path.getsize(file_name)
    #             premis_id = uuid.uuid4()
    #             premis_ids.append(premis_id)
    #             premis.append(
    #                 P.object(
    #                     {q(XSI_NS, 'type'): 'file', "xmlID":premis_id},
    #                     P.objectIdentifier(
    #                         P.objectIdentifierType('LOCAL'),
    #                         P.objectIdentifierValue(premis_id)
    #                     ),
    #                     P.objectIdentifier(
    #                         P.objectIdentifierType('FILEPATH'),
    #                         P.objectIdentifierValue(file_url)
    #                     ),
    #                     P.objectCharacteristics(
    #                         P.compositionLevel(0),
    #                         P.size(size),
    #                         P.fixity(
    #                             P.messageDigestAlgorithm("SHA-256"),
    #                             P.messageDigest(hash),
    #                             P.messageDigestOriginator("hashlib")
    #                         ),
    #                         P.format(
    #                             P.formatRegistry(
    #                                 P.formatRegistryName("PRONOM"),
    #                                 P.formatRegistryKey(fmt),
    #                                 P.formatRegistryRole("identification")
    #                             )
    #                         ),
    #                         #P.objectCharacteristicsExtension(
    #                             #TODO:// generate id or reference from somewhere
    #                         #    P.mdSec({"ID":"ID426087e8-0f79-11e3-847a-34e6d700c47b"},
    #                         #        P.mdWrap({"MDTYPE":"OTHER", "OTHERMDTYPE":"JHOVE"},
    #                         #            P.xmlData(
    #                         #                jhove
    #                         #                 )
    #                         #                 )
    #                         #)
    #                     ),
    #                 )
    #             )
    #
    #     identifier_value = 'earkweb'
    #     premis.append(P.agent(
    #             P.agentIdentifier(
    #                 P.agentIdentifierType('LOCAL'),
    #                 P.agentIdentifierValue(identifier_value)
    #             ),
    #             P.agentName('E-ARK AIP to DIP Converter'),
    #             P.agentType('Software')))
    #
    #     identifier_value = 'AIP Creation'
    #     linking_agent = 'earkweb'
    #     linking_object=None
    #     premis.append(P.event(
    #             P.eventIdentifier(
    #                 P.eventIdentifierType('LOCAL'),
    #                 P.eventIdentifierValue(identifier_value)
    #             ),
    #             P.eventType,
    #             P.eventDateTime(current_timestamp()),
    #             P.linkingAgentIdentifier(
    #                 P.linkingAgentIdentifierType('LOCAL'),
    #                 P.linkingAgentIdentifierValue(linking_agent)
    #             ),
    #
    #             P.linkingAgentIdentifier(
    #                 P.linkingAgentIdentifierType('LOCAL'),
    #                 P.linkingAgentIdentifierValue(linking_object)
    #             )
    #             if linking_object is not None else None
    #         ))
    #
    #     str = etree.tostring(premis, encoding='UTF-8', pretty_print=True, xml_declaration=True)
    #     preservation_dir = os.path.join(self.root_path, './metadata/preservation')
    #     if not os.path.exists(preservation_dir):
    #         os.mkdir(preservation_dir)
    #     path_premis = os.path.join(self.root_path, './metadata/preservation/premis.xml')
    #     with open(path_premis, 'w') as output_file:
    #         output_file.write(str)
    #
    #     return premis_ids

    # def createSIPParentMets(self, mets_files):
    #     #create METS skeleton
    #     METS_ATTRIBUTES = {"OBJID" : "", "TYPE" : "", "LABEL" : "", "PROFILE" : "http://www.ra.ee/METS/v01/IP.xml", "ID" : "" }
    #     root = M.mets(METS_ATTRIBUTES)
    #     root.attrib['{%s}schemaLocation' % XSI_NS] = "http://www.loc.gov/METS/ schemas/mets.xsd ExtensionMETS schemas/ExtensionMETS.xsd http://www.w3.org/1999/xlink schemas/xlink.xsd"
    #
    #     mets_hdr = M.metsHdr({"CREATEDATE": current_timestamp(), q(METSEXT_NS,"OAISSTATUS") :"", "RECORDSTATUS" :""})
    #     root.append(mets_hdr)
    #
    #
    #     mets_hdr.append(self.createAgent("ARCHIVIST", "ORGANIZATION", "" ,"Institution", "Note"))
    #     mets_hdr.append(self.createAgent("ARCHIVIST", "OTHER", "" ,"Institution", "Note"))
    #     mets_hdr.append(self.createAgent("CREATOR", "ORGANIZATION", "", "Institution", "Note"))
    #     mets_hdr.append(self.createAgent("CREATOR", "OTHER", "SOFTWARE", "E-ARK SIP Creator", "VERSION=0.0.1"))
    #     mets_hdr.append(self.createAgent("PRESERVATION", "ORGANIZATION", "", "Institution", "Note"))
    #     mets_hdr.append(M.metsDocumentID("METS.xml"))
    #
    #     mets_dmd = M.dmdSec({"ID":""})
    #     root.append(mets_dmd)
    #
    #     mets_amdSec = M.amdSec({"ID":"ID" + uuid.uuid1().__str__()})
    #     root.append(mets_amdSec)
    #
    #     mets_techmd = M.techMD({"ID":"ID" + uuid.uuid1().__str__()})
    #     mets_amdSec.append(mets_techmd)
    #     #for id in premis_ids:
    #     #    mets_mdref = M.mdRef({"LOCTYPE":"URL", "MDTYPE":"PREMIS:OBJECT", q(XLINK_NS,"href"):"./metadata/preservation/PREMIS.xml#"+id.__str__()})
    #     #    mets_techmd.append(mets_mdref)
    #
    #     mets_fileSec = M.fileSec()
    #     root.append(mets_fileSec)
    #
    #     mets_filegroup = M.fileGrp({"ID": "ID" + uuid.uuid1().__str__()})
    #     mets_fileSec.append(mets_filegroup)
    #
    #     #content_ids = self.addFiles(os.path.join(self.root_path, 'data'), mets_filegroup)
    #     #metadata_ids = self.addFiles(os.path.join(self.root_path, 'metadata'), mets_filegroup)
    #
    #     #mets_ids = []
    #     #for file in mets_files:
    #     #    print file
    #     #    mets_id = self.addFile(file, mets_filegroup)
    #     #    mets_ids.append(mets_id)
    #
    #
    #     # checking for cross-representation metadata:
    #     #package_root = self.root_path.rsplit('/', 2)
    #     #package_metadata = os.path.join(package_root[0], 'metadata')
    #     #ext_metadata_ids = self.addFiles(package_metadata, mets_filegroup)
    #
    #     mets_structmap = M.structMap({"ID": "", "TYPE":"", "LABEL":"Simple grouping"})
    #     root.append(mets_structmap)
    #
    #     mets_structmap_div = M.div({"ADMID":"", "LABEL":"Package", "DMDID" : ""})
    #     mets_structmap.append(mets_structmap_div)
    #
    #     mets_structmap_mets_div = M.div({"LABEL":"Representation Metadata"})
    #     mets_structmap_div.append(mets_structmap_mets_div)
    #
    #     for file in mets_files:
    #         file_url = "./%s" % os.path.relpath(file, self.root_path)
    #         mptr = M.mptr({"LOCTYPE": "URL", q(XLINK_NS, 'href'): file_url })
    #         mets_structmap_mets_div.append(mptr)
    #
    #     #for id in mets_ids:
    #     #    mptr = M.mptr({"FILEID": id})
    #     #    mets_structmap_mets_div.append(mptr)
    #     #    #fptr = M.fptr({"FILEID": id})
    #     #    #mets_structmap_mets_div.append(fptr)
    #
    #     #mets_structmap_content_div = M.div({"LABEL":"Content"})
    #     #mets_structmap_div.append(mets_structmap_content_div)
    #     #for id in content_ids:
    #     #    fptr = M.fptr({"FILEID": id})
    #     #    mets_structmap_content_div.append(fptr)
    #
    #     #mets_structmap_metadata_div = M.div({"LABEL":"Metadata"})
    #     #mets_structmap_div.append(mets_structmap_metadata_div)
    #     #for id in metadata_ids:
    #     #    fptr = M.fptr({"FILEID": id})
    #     #    mets_structmap_metadata_div.append(fptr)
    #     #for id in ext_metadata_ids:
    #     #    fptr = M.fptr({"FILEID": id})
    #     #    mets_structmap_metadata_div.append(fptr)
    #
    #     #my_mets.fileSec.append(M.fileGrp({'USE': 'submission'}))
    #     #my_mets.fileSec(M.fileGrp({'USE': 'submission'}))
    #     #mets_schema_file = os.path.join(root_dir, "sandbox/sipgenerator/resources/ENA_RK_TartuLV_141127/schemas/mets.xsd")
    #     #mets_schema = etree.parse(mets_schema_file)
    #     #mets_xsd = etree.XMLSchema(mets_schema)
    #
    #     str = etree.tostring(root, encoding='UTF-8', pretty_print=True, xml_declaration=True)
    #
    #     path_mets = os.path.join(self.root_path,'METS.xml')
    #     with open(path_mets, 'w') as output_file:
    #         output_file.write(str)


    # def createSIPMets(self, premis_ids = None):
    #     if premis_ids == None:
    #         premis_ids = self.createPremis()
    #
    #     #create METS skeleton
    #     # METS_ATTRIBUTES = {"OBJID" : "", "TYPE" : "", "LABEL" : "", "PROFILE" : "http://www.ra.ee/METS/v01/IP.xml", "ID" : "" }
    #     METS_ATTRIBUTES = {"OBJID": "placeholder123",
    #                        "LABEL": "METS file describing the SIP matching the OBJID.",
    #                        "PROFILE": "http://www.ra.ee/METS/v01/IP.xml",
    #                        "TYPE": "SIP"}
    #     root = M.mets(METS_ATTRIBUTES)
    #     root.attrib['{%s}schemaLocation' % XSI_NS] = "http://www.loc.gov/METS/ schemas/mets.xsd ExtensionMETS schemas/ExtensionMETS.xsd http://www.w3.org/1999/xlink schemas/xlink.xsd"
    #
    #     # mets_hdr = M.metsHdr({"CREATEDATE": current_timestamp(), q(METSEXT_NS,"OAISSTATUS") :"", "RECORDSTATUS" :""})
    #     mets_hdr = M.metsHdr({"CREATEDATE": current_timestamp(), "RECORDSTATUS" :"NEW"})
    #     root.append(mets_hdr)
    #
    #     mets_hdr.append(self.createAgent("ARCHIVIST", "ORGANIZATION", "" ,"Institution", "Note"))
    #     mets_hdr.append(self.createAgent("ARCHIVIST", "OTHER", "" ,"Institution", "Note"))
    #     mets_hdr.append(self.createAgent("CREATOR", "ORGANIZATION", "", "Institution", "Note"))
    #     mets_hdr.append(self.createAgent("CREATOR", "OTHER", "SOFTWARE", "E-ARK SIP Creator", "VERSION=0.0.1"))
    #     mets_hdr.append(self.createAgent("PRESERVATION", "ORGANIZATION", "", "Institution", "Note"))
    #     mets_hdr.append(M.metsDocumentID("METS.xml"))
    #
    #     mets_dmd = M.dmdSec({"ID": "ID" + uuid.uuid4().__str__()})
    #     root.append(mets_dmd)
    #     # this is how to add descriptive metadata entry
    #     #file_name = "../schemas/ead.xml"
    #     #file_url = unicode(os.path.join("",file_name), "utf-8")
    #     #checksum = self.sha256(file_name)
    #     #file_size = os.path.getsize(file_name)
    #     #mets_mdref= M.mdRef({"LOCTYPE":"URL", "MDTYPE":"EAD", "MIMETYPE":"text/xml", "CREATED":current_timestamp(), q(XLINK_NS,"type"):"simple", q(XLINK_NS,"href"):file_url, "CHECKSUMTYPE":"SHA-256", "CHECKSUM":file_checksum, "SIZE":file_size})
    #     #mets_dmd.append(mets_mdref)
    #
    #     mets_amdSec = M.amdSec({"ID":"ID" + uuid.uuid4().__str__()})
    #     root.append(mets_amdSec)
    #
    #     mets_techmd = M.techMD({"ID":"ID" + uuid.uuid4().__str__()})
    #     mets_amdSec.append(mets_techmd)
    #     #for id in premis_ids:
    #     #    mets_mdref = M.mdRef({"LOCTYPE":"URL", "MDTYPE":"PREMIS:OBJECT", q(XLINK_NS,"href"):"./metadata/preservation/PREMIS.xml#"+id.__str__()})
    #     #    mets_techmd.append(mets_mdref)
    #     #techmd_ids = self.addFiles(os.path.join(self.root_path, 'metadata/preservation'))
    #
    #
    #     mets_fileSec = M.fileSec()
    #     root.append(mets_fileSec)
    #
    #     # general filegroup
    #     mets_filegroup = M.fileGrp({"ID": "ID" + uuid.uuid4().__str__(), "USE": "general filegroup"})
    #     mets_fileSec.append(mets_filegroup)
    #
    #     # data, metadata and schemas - package level
    #     content_ids = self.addFiles(os.path.join(self.root_path, 'data'), mets_filegroup)
    #     metadata_ids = self.addFiles(os.path.join(self.root_path, 'metadata'), mets_filegroup)
    #     schema_ids = self.addFiles(os.path.join(self.root_path, 'schemas'), mets_filegroup)
    #
    #     # checking for cross-representation metadata and schemas:
    #     package_root = self.root_path.rsplit('/', 2)
    #     package_metadata = os.path.join(package_root[0], 'metadata')
    #     cross_metadata_ids = self.addFiles(package_metadata, mets_filegroup)
    #     package_schemas = os.path.join(package_root[0], 'schemas')
    #     cross_schemas_ids = self.addFiles(package_schemas, mets_filegroup)
    #
    #     # mets_structmap = M.structMap({"ID": "", "TYPE":"", "LABEL":"Simple grouping"})
    #     mets_structmap = M.structMap({"LABEL": "Simple SIP structuring", "TYPE": "logical"})
    #     root.append(mets_structmap)
    #
    #     mets_structmap_div = M.div({"ADMID":"", "LABEL":"Package", "DMDID" : ""})
    #     mets_structmap.append(mets_structmap_div)
    #
    #     # files in /data
    #     mets_structmap_content_div = M.div({"LABEL":"Content"})
    #     mets_structmap_div.append(mets_structmap_content_div)
    #     for id in content_ids:
    #         fptr = M.fptr({"FILEID": id})
    #         mets_structmap_content_div.append(fptr)
    #
    #     # files in /metadata and ./../../metadata
    #     mets_structmap_metadata_div = M.div({"LABEL":"Metadata"})
    #     mets_structmap_div.append(mets_structmap_metadata_div)
    #     for id in metadata_ids:
    #         fptr = M.fptr({"FILEID": id})
    #         mets_structmap_metadata_div.append(fptr)
    #     for id in cross_metadata_ids:
    #         fptr = M.fptr({"FILEID": id})
    #         mets_structmap_metadata_div.append(fptr)
    #
    #     # files in /schemas and ./../../schemas
    #     mets_structmap_schemas_div = M.div({"LABEL": "Schemas"})
    #     mets_structmap_div.append(mets_structmap_schemas_div)
    #     for id in schema_ids:
    #         fptr = M.fptr({"FILEID": id})
    #         mets_structmap_schemas_div.append(fptr)
    #     for id in cross_schemas_ids:
    #         fptr = M.fptr({"FILEID": id})
    #         mets_structmap_schemas_div.append(fptr)
    #
    #     #my_mets.fileSec.append(M.fileGrp({'USE': 'submission'}))
    #     #my_mets.fileSec(M.fileGrp({'USE': 'submission'}))
    #     #mets_schema_file = os.path.join(root_dir, "sandbox/sipgenerator/resources/ENA_RK_TartuLV_141127/schemas/mets.xsd")
    #     #mets_schema = etree.parse(mets_schema_file)
    #     #mets_xsd = etree.XMLSchema(mets_schema)
    #
    #     str = etree.tostring(root, encoding='UTF-8', pretty_print=True, xml_declaration=True)
    #
    #     path_mets = os.path.join(self.root_path,'METS.xml')
    #     with open(path_mets, 'w') as output_file:
    #         output_file.write(str)


    #def createAIPMets(self, premis_ids=None):
    # def createAIPMets(self, packageid):
    #     '''
    #     Create the AIP METS file.
    #
    #     @param premis_ids:
    #     @return:
    #     '''
    #     #premis_ids = None
    #
    #     #if premis_ids == None:
    #     #    premis_ids = self.createPremis()
    #
    #     # TODO: add metadata files to correct metadata sections!
    #
    #     # create METS skeleton
    #     METS_ATTRIBUTES = {"OBJID": packageid,
    #                        "LABEL": "METS file describing the AIP matching the OBJID.",
    #                        "PROFILE": "http://www.ra.ee/METS/v01/IP.xml",
    #                        "TYPE": "AIP"}
    #     root = M.mets(METS_ATTRIBUTES)
    #     root.attrib['{%s}schemaLocation' % XSI_NS] = "http://www.loc.gov/METS/ schemas/mets.xsd ExtensionMETS schemas/ExtensionMETS.xsd http://www.w3.org/1999/xlink schemas/xlink.xsd"
    #
    #     # mets_hdr = M.metsHdr({"CREATEDATE": current_timestamp(), q(METSEXT_NS,"OAISSTATUS") :"", "RECORDSTATUS" :"AIP"})
    #     mets_hdr = M.metsHdr({"CREATEDATE": current_timestamp(), "RECORDSTATUS" :"NEW"})
    #     root.append(mets_hdr)
    #
    #     mets_hdr.append(self.createAgent("ARCHIVIST", "ORGANIZATION", "" ,"Institution", "Note"))
    #     mets_hdr.append(self.createAgent("ARCHIVIST", "OTHER", "" ,"Institution", "Note"))
    #     mets_hdr.append(self.createAgent("CREATOR", "ORGANIZATION", "", "Institution", "Note"))
    #     mets_hdr.append(self.createAgent("CREATOR", "OTHER", "SOFTWARE", "E-ARK SIP Creator", "VERSION=0.0.1"))
    #     mets_hdr.append(self.createAgent("PRESERVATION", "ORGANIZATION", "", "Institution", "Note"))
    #     mets_hdr.append(M.metsDocumentID("METS.xml"))
    #
    #     mets_dmd = M.dmdSec({"ID": "ID" + uuid.uuid4().__str__()})
    #     root.append(mets_dmd)
    #     # this is how to add descriptive metadata entry
    #     #file_name = "../schemas/ead.xml"
    #     #file_url = unicode(os.path.join(",file_name), "utf-8")
    #     #checksum = self.sha256(file_name)
    #     #file_size = os.path.getsize(file_name)
    #     #mets_mdref= M.mdRef({"LOCTYPE":"URL", "MDTYPE":"EAD", "MIMETYPE":"text/xml", "CREATED":current_timestamp(), q(XLINK_NS,"type"):"simple", q(XLINK_NS,"href"):file_url, "CHECKSUMTYPE":"SHA-256", "CHECKSUM":file_checksum, "SIZE":file_size})
    #     #mets_dmd.append(mets_mdref)
    #
    #     # create amdSec
    #     mets_amdSec = M.amdSec({"ID": "ID" + uuid.uuid4().__str__()})
    #     root.append(mets_amdSec)
    #
    #     # create techMD
    #     #mets_techmd = M.techMD({"ID": "ID" + uuid.uuid4().__str__()})
    #     #mets_amdSec.append(mets_techmd)
    #     #for id in premis_ids:
    #     #    mets_mdref = M.mdRef({"LOCTYPE":"URL", "MDTYPE":"PREMIS:OBJECT", q(XLINK_NS,"href"):"./metadata/preservation/PREMIS.xml#"+id.__str__()})
    #     #    mets_techmd.append(mets_mdref)
    #
    #     mets_fileSec = M.fileSec()
    #     root.append(mets_fileSec)
    #
    #     # TODO: metadata entries into correct mets section + structmap: seperate between AIP and rep level metadata!
    #     # check if there is another METS file referencing the metadata
    #
    #     # general filegroup
    #     mets_filegroup = M.fileGrp({"ID": "ID" + uuid.uuid4().__str__(), "USE": "general filegroup"})
    #     mets_fileSec.append(mets_filegroup)
    #
    #     #metadata_ids = self.addFiles(os.path.join(self.root_path, 'metadata'), mets_filegroup)
    #     #submission_meta_ids = self.addFiles(os.path.join(self.root_path, 'submission/metadata'), mets_filegroup)
    #
    #     # mets_structmap = M.structMap({"ID": "", "TYPE":"", "LABEL":"Simple grouping"})
    #     mets_structmap = M.structMap({"LABEL": "Simple AIP structuring", "TYPE": "logical"})
    #     root.append(mets_structmap)
    #
    #     mets_structmap_div = M.div({"ADMID":"", "LABEL":"Package", "DMDID" : ""})
    #     mets_structmap.append(mets_structmap_div)
    #
    #     # metadata structmap - IP root level!
    #     mets_structmap_metadata_div = M.div({"LABEL": "Metadata"})
    #     mets_structmap_div.append(mets_structmap_metadata_div)
    #     #for id in metadata_ids:
    #     #    fptr = M.fptr({"FILEID": id})
    #     #    mets_structmap_metadata_div.append(fptr)
    #
    #     # metadata structmap - submission level!
    #     #mets_structmap_metadata_sub_div = M.div({"LABEL": "Metadata Submission"})
    #     #mets_structmap_div.append(mets_structmap_metadata_sub_div)
    #     #for id in submission_meta_ids:
    #     #    fptr = M.fptr({"FILEID": id})
    #     #    mets_structmap_metadata_sub_div.append(fptr)
    #
    #     # content structmap - all representations! (is only filled if no separate METS exists for the rep)
    #     mets_structmap_content_div = M.div({"LABEL": "various files"})
    #     mets_structmap_div.append(mets_structmap_content_div)
    #
    #     # structmap for schema files
    #     mets_structmap_schema_div = M.div({"LABEL": "schema files"})
    #     mets_structmap_div.append(mets_structmap_schema_div)
    #
    #     # create structmap for representations
    #     # mets_structmap_reps = M.structMap({"ID": "", "TYPE":"", "LABEL":"representations"})
    #     mets_structmap_reps = M.structMap({"TYPE":"logical", "LABEL":"representations"})
    #     root.append(mets_structmap_reps)
    #     mets_div_reps = M.div({'LABEL': 'representations', 'TYPE': 'type'})
    #     mets_structmap_reps.append(mets_div_reps)
    #
    #     # package
    #     workdir_length = len(self.root_path)
    #     #for directory, subdirectories, filenames in os.walk(os.path.join(self.root_path, 'submission/representations')):
    #     for directory, subdirectories, filenames in os.walk(self.root_path):
    #         if directory == os.path.join(self.root_path, 'metadata/earkweb'):
    #             del directory
    #             del filenames[:]
    #             del subdirectories[:]
    #         if len(filenames) > 0:
    #             for filename in filenames:
    #                 # ignore all files on AIP root level (since they wont be packed with the AIP):
    #                 if directory == self.root_path:
    #                     del filename
    #                 else:
    #                     # TODO: list rep metadata only in the rep Mets?
    #                     rel_path_file = ('.' + directory[workdir_length:] + '/' + filename).decode('utf-8')
    #                     if filename.lower() == 'mets.xml':
    #                         # delete the subdirectories list to stop os.walk from traversing further;
    #                         # mets file should be added as <mets:mptr> to <structMap> for corresponding rep
    #                         del subdirectories[:]
    #                         rep_name = directory.rsplit('/', 2)
    #                         rep_name = os.path.join(rep_name[1], rep_name[2])
    #                         # create structMap div and append to representations structMap
    #                         mets_structmap_rep_div = M.div({"LABEL": rep_name, "TYPE": "representation mets", "ID": "ID" + uuid.uuid4().__str__()})
    #                         mets_div_reps.append(mets_structmap_rep_div)
    #                         # add mets file as <mets:mptr>
    #                         metspointer = M.mptr({"LOCTYPE": "URL",
    #                                               q(XLINK_NS,"title"): "mets file describing representation: " + rep_name + " of AIP: " + packageid,
    #                                               q(XLINK_NS,"href"): rel_path_file})
    #                                              #"ID": uuid.uuid4().__str__()})
    #                         mets_structmap_rep_div.append(metspointer)
    #                         # also add the rep mets to the filegroup, so we can have a fptr
    #                         id = self.addFile(os.path.join(directory, filename), mets_filegroup)
    #                         mets_fptr = M.fptr({"FILEID": id})
    #                         mets_structmap_rep_div.append(mets_fptr)
    #                     elif filename and directory.endswith('schemas'):
    #                         # schema files
    #                         id = self.addFile(os.path.join(directory, filename), mets_filegroup)
    #                         fptr = M.fptr({'FILEID': id})
    #                         mets_structmap_schema_div.append(fptr)
    #                     elif filename == 'earkweb.log':
    #                         # earkweb log file - currently treated as digiprovMD
    #                         mets_digiprovmd = M.digiprovMD({"ID": "ID" + uuid.uuid4().__str__()})
    #                         mets_amdSec.append(mets_digiprovmd)
    #                         checksum = self.sha256(os.path.join(directory,filename))
    #                         id = "ID" + uuid.uuid4().__str__()
    #                         mets_mdref = M.mdRef({"LOCTYPE":"URL",
    #                                              "MIMETYPE":"text/xml",
    #                                              "CREATED":current_timestamp(),
    #                                              q(XLINK_NS,"type"):"simple",
    #                                              q(XLINK_NS,"href"):rel_path_file,
    #                                              "CHECKSUMTYPE":"SHA-256",
    #                                              "CHECKSUM":checksum,
    #                                              "ID": id,
    #                                               "MDTYPE": "OTHER"})
    #                         mets_digiprovmd.append(mets_mdref)
    #                         fptr = M.fptr({"FILEID": id})
    #                         mets_structmap_metadata_div.append(fptr)
    #                     elif directory.endswith('descriptive'):
    #                         # descriptive metadata
    #                         #mets_dmd = M.dmdSec({"ID": "ID" + uuid.uuid4().__str__()})
    #                         #root.append(mets_dmd)
    #                         checksum = self.sha256(os.path.join(directory,filename))
    #                         id = "ID" + uuid.uuid4().__str__()
    #                         mets_mdref = M.mdRef({"LOCTYPE":"URL",
    #                                              "MIMETYPE":"text/xml",
    #                                              "CREATED":current_timestamp(),
    #                                              q(XLINK_NS,"type"):"simple",
    #                                              q(XLINK_NS,"href"):rel_path_file,
    #                                              "CHECKSUMTYPE":"SHA-256",
    #                                              "CHECKSUM":checksum,
    #                                              "ID": id,
    #                                               "MDTYPE": "OTHER"})
    #                         mets_dmd.append(mets_mdref)
    #                         fptr = M.fptr({"FILEID": id})
    #                         mets_structmap_metadata_div.append(fptr)
    #                     elif directory.endswith('preservation'):
    #                         # preservation metadata (premis, techMD?)
    #                         #mets_techmd = M.techMD({"ID": id})
    #                         #mets_amdSec.append(mets_techmd)
    #                         mets_digiprovmd = M.digiprovMD({"ID": "ID" + uuid.uuid4().__str__()})
    #                         mets_amdSec.append(mets_digiprovmd)
    #                         checksum = self.sha256(os.path.join(directory,filename))
    #                         id = "ID" + uuid.uuid4().__str__()
    #                         mets_mdref = M.mdRef({"LOCTYPE":"URL",
    #                                              "MIMETYPE":"text/xml",
    #                                              "CREATED":current_timestamp(),
    #                                              q(XLINK_NS,"type"):"simple",
    #                                              q(XLINK_NS,"href"):rel_path_file,
    #                                              "CHECKSUMTYPE":"SHA-256",
    #                                              "CHECKSUM":checksum,
    #                                              "ID": id,
    #                                               "MDTYPE": "OTHER"})
    #                         #mets_techmd.append(mets_mdref)
    #                         mets_digiprovmd.append(mets_mdref)
    #                         fptr = M.fptr({"FILEID": id})
    #                         mets_structmap_metadata_div.append(fptr)
    #                     elif filename and not (directory.endswith('descriptive') or
    #                                            directory.endswith('metadata') or
    #                                            directory.endswith('preservation')):
    #                         # print 'found a file: ' + os.path.join(directory, filename)
    #                         id = self.addFile(os.path.join(directory, filename), mets_filegroup)
    #                         fptr = M.fptr({"FILEID": id})
    #                         mets_structmap_content_div.append(fptr)
    #
    #
    #     str = etree.tostring(root, encoding='UTF-8', pretty_print=True, xml_declaration=True)
    #
    #     path_mets = os.path.join(self.root_path,'METS.xml')
    #     with open(path_mets, 'w') as output_file:
    #         output_file.write(str)


    def createDeliveryMets(self, input_archive, output_mets):
        #create delivery METS skeleton
        METS_ATTRIBUTES = {"OBJID" : "UUID:" + uuid.uuid4().__str__(), "TYPE" : "SIP", "LABEL" : "Delivery METS", "PROFILE" : "http://webb.eark/package/METS/IP_CS.xml", "ID" : "ID" + uuid.uuid4().__str__() }
        root = M.mets(METS_ATTRIBUTES)
        root.attrib['{%s}schemaLocation' % XSI_NS] = "http://www.loc.gov/METS/ schemas/mets.xsd"

        mets_hdr = M.metsHdr({"CREATEDATE": current_timestamp()})
        root.append(mets_hdr)

        mets_hdr.append(self.createAgent("ARCHIVIST", "ORGANIZATION", "", "E-ARK", "Demonstrator"))
        mets_hdr.append(self.createAgent("CREATOR", "ORGANIZATION", "", "E-ARK", "Demonstrator"))
        mets_hdr.append(self.createAgent("CREATOR", "OTHER", "SOFTWARE", "E-ARK Web SIP Creator", "VERSION=1.0"))
        mets_hdr.append(self.createAgent("PRESERVATION", "ORGANIZATION", "", "E-ARK", "Demonstrator"))
        _,fname = os.path.split(output_mets)
        mets_hdr.append(M.metsDocumentID(fname))

        mets_fileSec = M.fileSec()
        root.append(mets_fileSec)

        mets_filegroup = M.fileGrp({"USE" : "PACKAGES", "ID": "ID" + uuid.uuid4().__str__()})
        mets_fileSec.append(mets_filegroup)

        content_id = self.addFile(input_archive, mets_filegroup)

        mets_structmap = M.structMap({"ID": "ID%s" % uuid.uuid4(), "TYPE": "physical", "LABEL": "Profilestructmap"})
        root.append(mets_structmap)
        mets_structmap_div = M.div({"LABEL": "Package"})
        mets_structmap.append(mets_structmap_div)
        mets_structmap_content_div = M.div({"LABEL": "Content"})
        mets_structmap_div.append(mets_structmap_content_div)
        fptr = M.fptr({"FILEID": "ID%s" % uuid.uuid4()})
        mets_structmap_content_div.append(fptr)

        xml_content = ET.tostring(root, encoding='UTF-8', pretty_print=True, xml_declaration=True)
        with open(output_mets, 'wb') as output_file:
            output_file.write(xml_content)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Command line utility for creating E-ARK Submission and Archival IPs. """
import argparse
import os
import uuid

# using absolute imports, therefore adding root to python path
import sys
sys.path.append(os.path.join(os.path.dirname(os.path.realpath(__file__)), "..")) # NOQA

from eatb import LOGGER
from eatb.oais_ip import create_sip, OAISPackageType, create_aip


def main():

    parser = argparse.ArgumentParser(description='eArchiving Toolbox - command line interface (CLI).')

    parser.add_argument('--name', "-n", type=str, help='Information package name', required=True)
    parser.add_argument('--directory', "-d", type=str, help='Information package content directory', required=True)
    parser.add_argument('--premis', "-p", action='store_true', help='Generate premis skeleton')
    parser.add_argument('--type', "-t", type=str, default="SIP",
                        choices=["SIP", "AIP"], help="Package type")
    parser.add_argument('--identifier', "-i", default=uuid.uuid4().__str__(), help='Identifier (default: generated)')
    parser.add_argument('--package', "-c", action='store_true', help='Create TAR package', default=True)
    args = parser.parse_args()

    if not os.path.exists(args.directory):
        msg = "The directory does not exist: %s" % args.directory
        LOGGER.error(msg)
        raise FileNotFoundError(msg)

    if not (os.path.isabs(args.directory)):
        msg = "The directory must be provided as absolute path: %s" % args.directory
        LOGGER.error(msg)
        raise ValueError(msg)

    if OAISPackageType[args.type] == OAISPackageType.SIP and create_sip(args.directory, args.name, args.identifier):
        LOGGER.info("SIP created successfully")
    elif OAISPackageType[args.type] == OAISPackageType.AIP and create_aip(args.directory, args.name, args.identifier):
        LOGGER.info("AIP created successfully")
    else:
        LOGGER.error("Package type not supported")


if __name__ == '__main__':
    main()



import os


def add_event(task, outcome, identifier_value,  linking_agent, package_premis_file, ip_work_dir):
    """
    Update premis file
    :param task:
    :param outcome:
    :param identifier_value:
    :param linking_agent:
    :param package_premis_file:
    :param ip_work_dir:
    :return:
    """
    package_premis_file.add_event(task, outcome, identifier_value, linking_agent)
    path_premis = os.path.join(ip_work_dir, "metadata/PREMIS.xml")
    with open(path_premis, 'w') as output_file:
        output_file.write(package_premis_file.to_string())
    print('PREMIS file updated: %s' % path_premis)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import random
import string
import uuid

def randomword(length):
    return ''.join(random.choice(string.ascii_lowercase) for _ in range(length))

def get_unique_id():
    return uuid.uuid4().__str__()



import re

camel_pat = re.compile(r'([A-Z])')
under_pat = re.compile(r'_([a-z])')


def camel_to_underscore(name):
    return camel_pat.sub(lambda x: '_' + x.group(1).lower(), name)


def underscore_to_camel(name):
    return under_pat.sub(lambda x: x.group(1).upper(), name)


def dict_keys_underscore_to_camel(d):
    d_new = {}
    for k, _ in d.items():
        d_new[underscore_to_camel(k)] = d[k]
    return d_new


def dict_keys_camel_to_underscore(d):
    d_new = {}
    for k, _ in d.items():
        d_new[camel_to_underscore(k)] = d[k]
    return d_new



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from xml.dom import minidom
from xml.etree.ElementTree import Element, SubElement
from xml.etree import ElementTree as Etree

from eatb.utils.xmlutils import prettify
from eatb.utils.datetime import current_timestamp


class IpState():
    """
    TaskExecutionXml class which represents an XML document to persist task execution parameters and results.
    The class can be initiated by parameters (static method from_parameters), by XML content string (static
    method from_content), or by an XML file path (static method from_path). Furthermore, it provides methods
    to manipulate and/or read element values of the XML document.
    """

    doc_content = None
    ted = None

    doc_path = None

    def __init__(self, doc_content, ted):
        self.doc_content = doc_content
        self.ted = ted

    @classmethod
    def from_content(cls, doc_content):
        """
        Alternative constructor (initialise from content string)

        @type doc_content: str
        @param doc_content: doc_content

        @rtype: TaskExecutionXml
        @return: TaskExecutionXml object
        """
        ted = Etree.fromstring(doc_content)
        return cls(doc_content, ted)

    @classmethod
    def from_path(cls, xml_file_path):
        """
        Alternative constructor (initialise from xml file)

        @type xml_file_path: str
        @param xml_file_path: xml_file_path

        @rtype: TaskExecutionXml
        @return: TaskExecutionXml object
        """
        with open(xml_file_path, 'r') as xml_file:
            doc_content = xml_file.read()
        ted = Etree.fromstring(doc_content)
        xml_file.close()
        return cls(doc_content, ted)

    @classmethod
    def from_parameters(cls, state=-1, locked_val=False, last_task_value='None'):
        """
        Alternative constructor (initialise from parameters)
        :param state: state
        :param locked_val: locked val
        :param last_task_value: last task value
        :return:
        """
        doc_content = prettify(cls.create_task_execution_doc(state, locked_val, last_task_value))
        ted = Etree.fromstring(doc_content)
        return cls(doc_content, ted)

    @classmethod
    def create_task_execution_doc(cls, state_val=-1, locked_val=False, last_task_value='None'):
        """
        Alternative constructor (initialise from parameters)
        :param state_val: state val
        :param locked_val: locked val
        :param last_task_value: last task value
        :return:
        """
        ip_state = Element('ip_state')
        state_elm = SubElement(ip_state, 'state')
        state_elm.text = str(state_val)
        locked_elm = SubElement(ip_state, 'locked')
        locked_elm.text = str(locked_val)
        last_task_elm = SubElement(ip_state, 'last_task')
        last_task_elm.text = last_task_value
        return ip_state

    def get_last_task(self):
        """
        Get last task
        :return: last task value
        """
        last_task_elm = self.ted.find('.//last_task')
        last_task_value = 'None' if last_task_elm is None else last_task_elm.text
        return last_task_value

    def set_last_task(self, last_task_value):
        """
        Set document path
        :param last_task_value: last task value
        :return: last task value
        """
        last_task_elm = self.ted.find('.//last_task')
        if last_task_elm is None:
            last_task_elm = SubElement(self.ted, 'last_task')
        last_task_elm.text = last_task_value

    def get_identifier(self):
        """
        Get identifier

        @rtype: str
        @return: identifier
        """
        identifier_elm = self.ted.find('.//identifier')
        identifier_value = 'None' if identifier_elm is None else identifier_elm.text
        return identifier_value

    def set_identifier(self, identifier_value):
        """
        Set identifier
        :param identifier_value: identifier value
        :return: identifier value
        """
        identifier_elm = self.ted.find('.//identifier')
        if identifier_elm is None:
            identifier_elm = SubElement(self.ted, 'identifier')
        identifier_elm.text = identifier_value

    def get_version(self):
        """
        Get version
        :return: version
        """
        version_elm = self.ted.find('.//version')
        version_value = '00000' if version_elm is None else version_elm.text
        return version_value

    def set_version(self, version_value):
        """
        Set version
        :param version_value: version
        :return: version
        """
        version_elm = self.ted.find('.//version')
        if version_elm is None:
            version_elm = SubElement(self.ted, 'version')
        version_elm.text = version_value

    def get_doc_path(self):
        """
        Get document path
        :return: document path
        """
        return self.doc_path

    def set_doc_path(self, doc_path):
        """
        Set document path
        :param doc_path: document path
        :return: document path
        """
        self.doc_path = doc_path

    def get_state(self):
        """
        Get state value.
        :return: state
        """
        return int(self.ted.find('.//state').text)

    def set_state(self, state_value):
        """
        Set state value
        :param state_value: state value
        :return: state value
        """
        state_elm = self.ted.find('.//state')
        if state_elm is None:
            state_elm = SubElement(self.ted, 'state')
        state_elm.text = str(state_value)

    def get_locked(self):
        """
        Get locked status
        :return: locked status
        """
        return self.ted.find('.//locked').text == "True"

    def set_locked(self, locked_value):
        """
        Set locked status
        :param locked_value:
        :return:
        """
        locked_elm = self.ted.find('.//locked')
        if locked_elm is None:
            locked_elm = SubElement(self.ted, 'locked')
        locked_elm.text = str(locked_value)

    def get_lastchange(self):
        """
        Get last change
        :return: last change
        """
        lastchange_elm = self.ted.find('.//lastchange')
        if lastchange_elm is None:
            return ""
        return self.ted.find('.//lastchange').text

    def set_lastchange(self, lastchange_value):
        """
        Set last change
        :param lastchange_value: last change
        :return: last change
        """
        lastchange_elm = self.ted.find('.//lastchange')
        if lastchange_elm is None:
            lastchange_elm = SubElement(self.ted, 'lastchange')
        lastchange_elm.text = str(lastchange_value)

    def get_updated_doc_content(self):
        """
        Get updated document content (from task execution document)
        :return: updated document content
        """
        return Etree.tostring(self.ted, encoding='UTF-8')

    def write_doc(self, xml_file_path):
        """
        Write document to file
        :param xml_file_path: xml file path
        :return: None
        """
        # update timestamp
        self.set_lastchange(current_timestamp())
        xmlstr = minidom.parseString(Etree.tostring(self.ted)).toprettyxml(indent="\t", newl="\n", encoding="UTF-8")
        with open(xml_file_path, 'w') as output_file:
            output_file.write(xmlstr.decode("utf-8"))
        output_file.close()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="eatb",
    version="0.2.4",
    author="E-ARK Foundation",
    author_email="admin@e-ark-foundation.eu",
    license='MIT',
    description="A suite of tools for the creation of information packages for archival purposes.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://www.e-ark-foundation.eu/e-ark-software-eatb",
    packages=setuptools.find_packages(),
    package_data={'eatb': ["*.ini", "**/*.cfg", "**/**/*.xsd", "**/*.xml"]},
    zip_safe=False,
    classifiers=[
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
          'Programming Language :: Python :: 3.9',
          'Programming Language :: Python :: 3.10',
          'Topic :: System :: Archiving',
    ],

)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import unittest

from eatb import ROOT
from eatb.metadata.parsed_dcat import ParsedDcat
from eatb.utils.fileutils import read_file_content


class TestParsedDcat(unittest.TestCase):
    test_dir = os.path.join(ROOT, 'tests/test_resources/')

    def test_get_dataset_property_value(self):
        """
        Test get dataset property value
        """
        xml_content = read_file_content(os.path.join(TestParsedDcat.test_dir, 'metadata/dcat/dcat.xml'))
        pead = ParsedDcat(xml_content.encode('utf-8'))
        title = pead.get_dataset_property_value("dct:title")
        print(title)

    def test_get_dataset_property_values(self):
        """
        Test get dataset property values
        """
        xml_content = read_file_content(
            os.path.join(TestParsedDcat.test_dir, TestParsedDcat.test_dir + 'metadata/dcat/dcat.xml'))
        pead = ParsedDcat(xml_content.encode('utf-8'))
        titles = pead.get_dataset_property_values(
            ["dct:title", "dct:description", "dcat:keyword", "dcat:theme", "dmav:User_Generated_Tags/dcat:keyword",
             "dct:publisher/foaf:Organization/foaf:name", "dcat:contactPoint/vcard:Organization/vcard:fn"]
        )
        print(titles)

    def test_get_distribution_property_values(self):
        """
        Test get dataset property values
        """
        xml_content = read_file_content(
            os.path.join(TestParsedDcat.test_dir + 'metadata/dcat/dcat.xml'))
        pead = ParsedDcat(xml_content.encode('utf-8'))
        titles = pead.get_distribution_property_values()
        print(titles)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-


def flatten_list(nested_list):
    for i in nested_list:
        if isinstance(i, (list, tuple)):
            for j in flatten_list(i):
                yield j
        else:
            yield i



from lxml import objectify
import logging

METS_NS = 'http://www.loc.gov/METS/'
METSEXT_NS = 'ExtensionMETS'
XLINK_NS = "http://www.w3.org/1999/xlink"
CSIP_NS = "https://DILCIS.eu/XML/METS/CSIPExtensionMETS"
METS_NSMAP = {None: METS_NS, "csip": CSIP_NS, "xlink": "http://www.w3.org/1999/xlink", "ext": METSEXT_NS,
              "xsi": "http://www.w3.org/2001/XMLSchema-instance"}
DELIVERY_METS_NSMAP = {None: METS_NS, "csip": CSIP_NS, "xlink": "http://www.w3.org/1999/xlink",
                       "xsi": "http://www.w3.org/2001/XMLSchema-instance"}
PROFILE_XML = "https://earkcsip.dilcis.eu/profile/E-ARK-CSIP.xml"

default_mets_schema_location = 'schemas/mets.xsd'
default_csip_location = "schemas/DILCISExtensionMETS.xsd"
default_xlink_schema_location = 'https://www.w3.org/1999/xlink.xsd'

M = objectify.ElementMaker(
    annotate=False,
    namespace=METS_NS,
    nsmap=METS_NSMAP)

folders_with_USE = ['documentation', 'schemas', 'representations', "data"]

logger = logging.getLogger(__name__)


import fnmatch
import os
import uuid
from mimetypes import MimeTypes
from subprocess import PIPE, Popen

from lxml import etree

from eatb.checksum import get_sha256_hash
from eatb.file_format import FormatIdentification
from eatb.metadata import logger, M, CSIP_NS, PROFILE_XML, default_mets_schema_location, \
    default_xlink_schema_location, default_csip_location, folders_with_USE
from eatb.metadata.mets import get_folder_with_USE
from eatb.metadata.mets_validation import XLINK_NS, METS_NS
from eatb.settings import application_name, application_version
from eatb.utils.XmlHelper import q, XSI_NS
from eatb.utils.datetime import get_file_ctime_iso_date_str, DT_ISO_FMT_SEC_PREC, current_timestamp


class MetsGenerator(object):
    '''
    This class generates a Mets file.
    It has to be instantiated (something = MetsGenerator(path) with the (A/S)IP root path as an argument (to specify
    the Mets directory; all subfolders will be treated as part of the IP. After this. the createMets can be called
    (something.createMets(data)) with a dictionary that must contain 'packageid', 'schemas' (location of the schema
    folder) and 'type', where 'type' must comply with the Mets standard for TYPE attribute of the Mets root.
    '''

    fid = FormatIdentification()
    mime = MimeTypes()
    root_path = ""
    mets_data = None

    def __init__(self, root_path):
        self.root_path = root_path

    def runCommand(self, program, stdin=PIPE, stdout=PIPE, stderr=PIPE):
        result, res_stdout, res_stderr = None, None, None
        try:
            # quote the executable otherwise we run into troubles
            # when the path contains spaces and additional arguments
            # are presented as well.
            # special: invoking bash as login shell here with
            # an unquoted command does not execute /etc/profile

            logger.debug('Launching: ' + ' '.join(program))
            process = Popen(program, stdin=stdin, stdout=stdout, stderr=stderr, shell=False)

            res_stdout, res_stderr = process.communicate()
            result = process.returncode
            logger.debug('Finished: ' + ' '.join(program))

        except Exception as ex:
            res_stderr = ''.join(str(ex.args))
            result = 1

        if result != 0:
            logger.debug('Command failed:' + ''.join(res_stderr))
            raise Exception('Command failed:' + ''.join(res_stderr))

        return result, res_stdout, res_stderr

    def createAgent(self,role, type, other_type, name, note, notetype=None):
        if other_type:
#            agent = M.agent({"ROLE": role, "TYPE": type, "OTHERTYPE": other_type}, M.name(name), M.note({q(CSIP_NS, "NOTETYPE"): notetype}, note))
            agent = M.agent({"ROLE": role, "TYPE": type, "OTHERTYPE": other_type}, M.name(name), M.note(note, {q(CSIP_NS, "NOTETYPE"): "SOFTWARE VERSION"}))
        else:
            agent = M.agent({"ROLE": role, "TYPE": type}, M.name(name), M.note(note), M.note(note, {q(CSIP_NS, "NOTETYPE"): "SOFTWARE VERSION"}))
        return agent

    def addFile(self, file_name, mets_filegroup):
        # reload(sys)
        # sys.setdefaultencoding('utf8')
        file_url = "%s" % os.path.relpath(file_name, self.root_path)
        file_mimetype, _ = self.mime.guess_type(file_url)
        file_mimetype = file_mimetype if file_mimetype else "application/octet-stream"
        file_checksum = get_sha256_hash(file_name)
        file_size = os.path.getsize(file_name)
        file_cdate = get_file_ctime_iso_date_str(file_name, DT_ISO_FMT_SEC_PREC)
        file_id = "ID" + uuid.uuid4().__str__()
        mets_file = M.file(
            {"MIMETYPE": file_mimetype, "CHECKSUMTYPE": "SHA-256", "CREATED": file_cdate, "CHECKSUM": file_checksum,
             "USE": "Datafile", "ID": file_id, "SIZE": file_size})
        mets_filegroup.append(mets_file)
        # _,fname = os.path.split(file_name)
        mets_FLocat = M.FLocat({q(XLINK_NS, 'href'): file_url, "LOCTYPE": "URL", q(XLINK_NS, 'type'): 'simple'})
        mets_file.append(mets_FLocat)
        return file_id

    def addFiles(self, folder, mets_filegroup):
        ids = []
        for top, dirs, files in os.walk(folder):
            for fn in files:
                file_name = os.path.join(top, fn)
                file_id = self.addFile(file_name, mets_filegroup)
                ids.append(file_id)
        return ids

    def make_mdref(self, path, file, id, mdtype):
        mimetype, _ = self.mime.guess_type(os.path.join(path, file))
        mimetype = mimetype if mimetype else "application/octet-stream"
        rel_path = "%s" % os.path.relpath(os.path.join(path, file), self.root_path)
        mets_mdref = {"LOCTYPE": "URL",
                      "MIMETYPE": mimetype,
                      "CREATED": current_timestamp(),
                      q(XLINK_NS, "type"): "simple",
                      q(XLINK_NS, "href"): rel_path,
                      "CHECKSUMTYPE": "SHA-256",
                      "CHECKSUM": get_sha256_hash(os.path.join(path, file)),
                      "ID": id,
                      "SIZE": os.path.getsize(os.path.join(path, file)),
                      "MDTYPE": mdtype}
        return mets_mdref

    def setParentRelation(self, identifier):
        parentmets = os.path.join(self.root_path, 'METS.xml')
        packagetype = self.mets_data['type']
        if os.path.exists(parentmets):
            parser = etree.XMLParser(resolve_entities=False, remove_blank_text=True, strip_cdata=False)
            parent_parse = etree.parse(parentmets, parser)
            parent_root = parent_parse.getroot()

            parent = M.div({'LABEL': "parent %s" % packagetype})
            pointer = M.mptr({"LOCTYPE": "OTHER",
                              "OTHERLOCTYPE": "UUID",
                              q(XLINK_NS, "title"): ("Referencing a parent %s." % packagetype),
                              q(XLINK_NS, "href"): identifier,
                              "ID": "ID" + uuid.uuid4().__str__()})
            parent.append(pointer)

            parent_map = parent_root.find("%s[@LABEL='parent %s']" % (q(METS_NS, 'structMap'), packagetype))
            if parent_map is not None:
                parent_div = parent_map.find("%s[@LABEL='parent %s identifiers']" % (q(METS_NS, 'div'), packagetype))
                parent_div.append(parent)
            else:
                parent_map = M.structMap({'LABEL': 'parent %s' % packagetype, 'TYPE': 'logical'})
                parent_div = M.div({'LABEL': 'parent %s identifiers' % packagetype})
                parent_map.append(parent_div)
                parent_div.append(parent)
                parent_root.insert(len(parent_root), parent_map)

            str = etree.tostring(parent_root, encoding='UTF-8', pretty_print=True, xml_declaration=True)
            with open(parentmets, 'w') as output_file:
                output_file.write(str)
        else:
            logger.debug('Couldn\'t find the parent %ss Mets file.' % packagetype)

    def addChildRelation(self, identifier):
        parentmets = os.path.join(self.root_path, 'METS.xml')
        packagetype = self.mets_data['type']
        if os.path.exists(parentmets):
            parser = etree.XMLParser(resolve_entities=False, remove_blank_text=True, strip_cdata=False)
            parent_parse = etree.parse(parentmets, parser)
            parent_root = parent_parse.getroot()
            child = M.div({'LABEL': "child %s" % packagetype})
            pointer = M.mptr({"LOCTYPE": "OTHER",
                              "OTHERLOCTYPE": "UUID",
                              q(XLINK_NS, "title"): ("Referencing a child %s." % packagetype),
                              q(XLINK_NS, "href"): identifier,
                              "ID": "ID" + uuid.uuid4().__str__()})
            child.append(pointer)

            children_map = parent_root.find("%s[@LABEL='child %s']" % (q(METS_NS, 'structMap'), packagetype))
            if children_map is not None:
                children_div = children_map.find("%s[@LABEL='child %s identifiers']" % (q(METS_NS, 'div'), packagetype))
                children_div.append(child)
            else:
                children_map = M.structMap({'LABEL': 'child %s' % packagetype, 'TYPE': 'logical'})
                children_div = M.div({'LABEL': 'child %s identifiers' % packagetype})
                children_map.append(children_div)
                children_div.append(child)
                parent_root.insert(len(parent_root), children_map)

            str = etree.tostring(parent_root, encoding='UTF-8', pretty_print=True, xml_declaration=True)
            with open(parentmets, 'w') as output_file:
                output_file.write(str)
        else:
            logger.debug('Couldn\'t find the parent %ss Mets file.' % packagetype)

    def createMets(self, mets_data, mets_file_path=None):
        self.mets_data = mets_data
        packageid = mets_data['packageid']
        packagetype = mets_data['type']
        schemafolder = mets_data['schemas']
        parent = mets_data['parent']

        ###########################
        # create METS skeleton
        ###########################

        # create Mets root
        METS_ATTRIBUTES = {"OBJID": packageid,
                           "LABEL": "METS file describing the %s matching the OBJID." % packagetype,
                           "PROFILE": PROFILE_XML,
                           "TYPE": "Databases",
                           q(CSIP_NS, "CONTENTINFORMATIONTYPE"): "SIARD2"}
        root = M.mets(METS_ATTRIBUTES)

        if os.path.isfile(os.path.join(schemafolder, 'mets.xsd')):
            mets_schema_location = os.path.relpath(os.path.join(schemafolder, 'mets.xsd'), self.root_path)
        else:
            mets_schema_location = default_mets_schema_location
        if os.path.isfile(os.path.join(schemafolder, 'xlink.xsd')):
            xlink_schema_location = os.path.relpath(os.path.join(schemafolder, 'xlink.xsd'), self.root_path)
        else:
            xlink_schema_location = default_xlink_schema_location

        root.attrib['{%s}schemaLocation' % XSI_NS] = "%s %s " \
                                                     "%s %s " \
                                                     "%s %s" % \
                                                     (METS_NS, mets_schema_location,
                                                      XLINK_NS, xlink_schema_location,
                                                      CSIP_NS, default_csip_location)

        # create Mets header
        mets_hdr = M.metsHdr({"CREATEDATE": current_timestamp(), "RECORDSTATUS": "NEW", q(CSIP_NS, "OAISPACKAGETYPE"): packagetype})
        root.append(mets_hdr)

        # add an agent
        mets_hdr.append(self.createAgent("CREATOR", "OTHER", "SOFTWARE", application_name, "VERSION=%s" % application_version, "SOFTWARE VERSION"))
        mets_hdr.append(M.agent({"ROLE": "ARCHIVIST", "TYPE": "ORGANIZATION"}, M.name("E-ARK")))
        mets_hdr.append(M.agent({"ROLE": "CREATOR", "TYPE": "ORGANIZATION"}, M.name("E-ARK")))
        mets_hdr.append(M.agent({"ROLE": "PRESERVATION", "TYPE": "ORGANIZATION"}, M.name("E-ARK")))

        # add document ID
        mets_hdr.append(M.metsDocumentID("METS.xml"))

        # create amdSec
        mets_amdSec = M.amdSec({"ID": "ID" + uuid.uuid4().__str__()})
        root.append(mets_amdSec)

        # create fileSec
        mets_fileSec = M.fileSec({"ID": "ID" + uuid.uuid4().__str__()})
        root.append(mets_fileSec)

        # filegroups
        mets_filegroups = dict()

        folder_with_USE = get_folder_with_USE(self.root_path)
        mets_filegroups[folder_with_USE] = M.fileGrp({"ID": "ID" + uuid.uuid4().__str__(), "USE": folder_with_USE})
        # for METSs in subfolders
        for subdir in next(os.walk(self.root_path))[1]:
            if subdir in folders_with_USE:
                mets_filegroups[subdir] = M.fileGrp({"ID": "ID" + uuid.uuid4().__str__(), "USE": subdir})
                mets_fileSec.append(mets_filegroups[subdir])

        # structMap 'CSIP' - default, physical structure
        mets_earkstructmap = M.structMap({"LABEL": "CSIP", "TYPE": "PHYSICAL", "ID": "ID" + uuid.uuid4().__str__()})
        root.append(mets_earkstructmap)
        package_div = M.div({"LABEL": packageid, "ID": "ID" + uuid.uuid4().__str__()})
        # append physical structMap
        mets_earkstructmap.append(package_div)

        # structMap and div for the whole package (metadata, schema and /data)
        mets_structmap = M.structMap({"LABEL": "Simple %s structuring" % packagetype, "TYPE": "logical", "ID": "ID" + uuid.uuid4().__str__()})
        root.append(mets_structmap)
        mets_structmap_div = M.div({"LABEL": "Package structure", "ID": "ID" + uuid.uuid4().__str__()})
        mets_structmap.append(mets_structmap_div)

        # metadata structmap - IP root level!
        mets_structmap_metadata_div = M.div({"LABEL": "metadata files", "ID": "ID" + uuid.uuid4().__str__()})
        mets_structmap_div.append(mets_structmap_metadata_div)

        # structmap for schema files
        mets_structmap_schema_div = M.div({"LABEL": "schema files", "ID": "ID" + uuid.uuid4().__str__()})
        mets_structmap_div.append(mets_structmap_schema_div)

        # content structmap - all representations! (is only filled if no separate METS exists for the rep)
        mets_structmap_content_div = M.div({"LABEL": "content files", "ID": "ID" + uuid.uuid4().__str__()})
        mets_structmap_div.append(mets_structmap_content_div)

        # create structmap and div for Mets files from representations
        # mets_structmap_reps = M.structMap({"TYPE": "logical", "LABEL": "representations"})
        # root.append(mets_structmap_reps)
        # mets_div_reps = M.div({"LABEL": "representations", "TYPE": "type"})
        # mets_structmap_reps.append(mets_div_reps)

        # create structmap for parent/child relation, if applicable
        if parent and parent != '':
            logger.debug('creating link to parent %s' % packagetype)
            mets_structmap_relation = M.structMap({'TYPE': 'logical', 'LABEL': 'parent'})
            root.append(mets_structmap_relation)
            mets_div_rel = M.div({'LABEL': '%s parent identifier' % packagetype, "ID": "ID" + uuid.uuid4().__str__()})
            mets_structmap_relation.append(mets_div_rel)
            parent_pointer = M.mptr({"LOCTYPE": "OTHER",
                                     "OTHERLOCTYPE": "UUID",
                                     q(XLINK_NS, "title"): ("Referencing the parent %s of this (%s) %s." % (packagetype, packageid, packagetype)),
                                     q(XLINK_NS, "href"): parent,
                                     "ID": "ID" + uuid.uuid4().__str__()})
            mets_div_rel.append(parent_pointer)

        ###########################
        # add to Mets skeleton
        ###########################

        # add the package content to the Mets skeleton
        for directory, subdirectories, filenames in os.walk(self.root_path):
            folder_with_USE = get_folder_with_USE(directory)
            # build the earkstructmap
            path = os.path.relpath(directory, self.root_path)
            physical_div = ''
            if path != 'representations': # otherwise an empty div is created
                if path != '.':
                    physical_div = M.div({"LABEL": path, "ID": "ID" + uuid.uuid4().__str__()})
                    package_div.append(physical_div)
                # if directory.endswith('metadata/conduit'):
                #     # Ignore temp files only needed for IP processing with conduit
                #     del filenames[:]
                #     del subdirectories[:]
                if directory.endswith('/metadata'):
                    del filenames[:]
                    del subdirectories[:]
                if directory == os.path.join(self.root_path, 'metadata'):
                    # Metadata on IP root level - if there are folders for representation-specific metadata,
                    # check if the corresponding representation has a Mets file. If yes, skip; if no, add to IP root Mets.
                    for filename in filenames:
                        if filename == 'conduit.log':
                            mets_digiprovmd = M.digiprovMD({"ID": "ID" + uuid.uuid4().__str__()})
                            mets_amdSec.append(mets_digiprovmd)
                            id = "ID" + uuid.uuid4().__str__()
                            ref = self.make_mdref(directory, filename, id, 'OTHER')
                            mets_mdref = M.mdRef(ref)
                            mets_digiprovmd.append(mets_mdref)
                            mets_structmap_metadata_div.append(M.fptr({"FILEID": id}))
                            physical_div.append(M.fptr({"FILEID": id}))
                    del subdirectories[:]  # prevent loop to iterate subfolders outside of this if statement
                    dirlist = os.listdir(os.path.join(self.root_path, 'metadata'))
                    for dirname in dirlist:
                        if fnmatch.fnmatch(dirname, '*_mig-*'):
                            # TODO: maybe list it all the time?
                            # this folder contains metadata for a representation/migration, currently:
                            # only listed if no representation Mets file exists
                            if os.path.isfile(os.path.join(self.root_path, 'representations/%s/METS.xml') % dirname):
                                pass
                            else:
                                for dir, subdir, files in os.walk(os.path.join(self.root_path, 'metadata/%s') % dirname):
                                    for filename in files:
                                        if dir.endswith('descriptive'):
                                            mets_dmd = M.dmdSec({"ID": "ID" + uuid.uuid4().__str__(), "CREATED": current_timestamp(), "STATUS": "CURRENT"})
                                            root.insert(1, mets_dmd)
                                            id = "ID" + uuid.uuid4().__str__()
                                            ref = self.make_mdref(dir, filename, id, 'OTHER')
                                            mets_mdref = M.mdRef(ref)
                                            mets_dmd.append(mets_mdref)
                                            mets_structmap_metadata_div.append(M.fptr({"FILEID": id}))
                                            physical_div.append(M.fptr({"FILEID": id}))
                                        elif dir.endswith('preservation'):
                                            mets_digiprovmd = M.digiprovMD({"ID": "ID" + uuid.uuid4().__str__()})
                                            mets_amdSec.append(mets_digiprovmd)
                                            id = "ID" + uuid.uuid4().__str__()
                                            mdtype = ''
                                            if filename.startswith('premis') or filename.endswith('premis.xml'):
                                                mdtype = 'PREMIS'
                                            else:
                                                mdtype = 'OTHER'
                                            ref = self.make_mdref(dir, filename, id, mdtype)
                                            mets_mdref = M.mdRef(ref)
                                            mets_digiprovmd.append(mets_mdref)
                                            mets_structmap_metadata_div.append(M.fptr({"FILEID": id}))
                                            physical_div.append(M.fptr({"FILEID": id}))
                                        elif filename:
                                            logger.debug('Unclassified metadata file %s in %s.' % (filename, dir))
                        else:
                            # metadata that should be listed in the Mets
                            for dir, subdir, files in os.walk(os.path.join(self.root_path, 'metadata/%s') % dirname):
                                if len(files) > 0:
                                    for filename in files:
                                        #if dir.endswith('descriptive'):
                                        if dirname == 'descriptive':
                                            mets_dmd = M.dmdSec({"ID": "ID" + uuid.uuid4().__str__(), "CREATED": current_timestamp(), "STATUS": "CURRENT"})
                                            root.insert(1, mets_dmd)
                                            id = "ID" + uuid.uuid4().__str__()
                                            # TODO: change MDTYPE
                                            ref = self.make_mdref(dir, filename, id, 'OTHER')
                                            mets_mdref = M.mdRef(ref)
                                            mets_dmd.append(mets_mdref)
                                            mets_structmap_metadata_div.append(M.fptr({"FILEID": id}))
                                            physical_div.append(M.fptr({"FILEID": id}))
                                        #elif dir.endswith('preservation'):
                                        elif dirname == 'preservation' or dirname == 'conduit':
                                            mets_digiprovmd = M.digiprovMD({"ID": "ID" + uuid.uuid4().__str__(), "STATUS": "CURRENT"})
                                            mets_amdSec.append(mets_digiprovmd)
                                            id = "ID" + uuid.uuid4().__str__()
                                            mdtype = ''
                                            if filename.startswith('premis') or filename.endswith('premis.xml'):
                                                mdtype = 'PREMIS'
                                            elif filename:
                                                mdtype = 'OTHER'
                                            ref = self.make_mdref(dir, filename, id, mdtype)
                                            mets_mdref = M.mdRef(ref)
                                            mets_digiprovmd.append(mets_mdref)
                                            mets_structmap_metadata_div.append(M.fptr({"FILEID": id}))
                                            physical_div.append(M.fptr({"FILEID": id}))
                                        elif filename:
                                            logger.debug('Unclassified metadata file %s in %s.' % (filename, dir))
                else:
                    # Any other folder outside of /<root>/metadata
                    for filename in filenames:
                        if directory == self.root_path:
                            # ignore files on IP root level
                            del filename
                        else:
                            # TODO: list rep metadata only in the rep Mets?
                            rel_path_file = "%s" % os.path.relpath(os.path.join(directory, filename), self.root_path)
                            if filename.lower() == 'mets.xml':
                                # delete the subdirectories list to stop os.walk from traversing further;
                                # mets file should be added as <mets:mptr> to <structMap> for corresponding rep
                                del subdirectories[:]
                                rep_name = directory.rsplit('/', 1)[1]
                                # create structMap div and append to representations structMap
                                # mets_structmap_rep_div = M.div({"LABEL": rep_name, "TYPE": "representation mets", "ID": "ID" + uuid.uuid4().__str__()})
                                # mets_div_reps.append(mets_structmap_rep_div)
                                # add mets file as <mets:mptr>
                                metspointer = M.mptr({"LOCTYPE": "URL",
                                                      q(XLINK_NS, "title"): ("Mets file describing representation: %s of %s: urn:uuid:%s." % (rep_name, packagetype, packageid)),
                                                      q(XLINK_NS, "href"): rel_path_file,
                                                      q(XLINK_NS, 'type'): 'simple',
                                                      "ID": "ID" + uuid.uuid4().__str__()})
                                #mets_structmap_rep_div.append(metspointer)
                                #mets_structmap_rep_div.append(M.fptr({"FILEID": id}))
                                physical_div.append(metspointer)    # IMPORTANT: The <mptr> element needs to be the first entry in a <div>, or the Mets will be invalid!
                                # also create a <fptr> for the Mets file
                                id = self.addFile(os.path.join(directory, filename), mets_filegroups[folder_with_USE])
                                physical_div.append(M.fptr({"FILEID": id}))
                            elif filename and directory.endswith('schemas'):
                                # schema files
                                id = self.addFile(os.path.join(directory, filename), mets_filegroups[folder_with_USE])
                                mets_structmap_schema_div.append(M.fptr({'FILEID': id}))
                                physical_div.append(M.fptr({'FILEID': id}))
                            elif filename:
                                try:
                                    id = self.addFile(os.path.join(directory, filename),
                                                      mets_filegroups[folder_with_USE])
                                    mets_structmap_content_div.append(M.fptr({'FILEID': id}))
                                    physical_div.append(M.fptr({'FILEID': id}))
                                except KeyError as error:
                                    logger.error("Error looking up '%s' element for file '%s'" % (folder_with_USE, filename), error)

        str = etree.tostring(root, encoding='UTF-8', pretty_print=True, xml_declaration=True)

        if not mets_file_path:
            mets_file_path = os.path.join(self.root_path, 'METS.xml')
        with open(mets_file_path, 'w') as output_file:
            output_file.write(str.decode('utf-8'))





#!/usr/bin/env python
# -*- coding: utf-8 -*-
from datetime import datetime
import os
import pytz
from pytz import timezone
import time


DT_ISO_FORMAT = '%Y-%m-%dT%H:%M:%SZ'

DT_ISO_FMT_SEC_PREC = '%Y-%m-%dT%H:%M:%S'

TS_FORMAT = '%Y-%m-%d %H:%M:%S'

EU_UI_FORMAT = '%d.%m.%Y %H:%M:%S'

DATE_DMY = '%d.%m.%Y'

DT_ISO_FORMAT_FILENAME = "%Y%m%d-%H%M%SZ"

TASK_EXEC_TIMES = {}


class LengthBasedDateFormat(object):
    date_str = None

    def __init__(self, date_str):
        self.date_str = date_str

    def reformat(self, target_fmt='%Y-%m-%dT%H:%M:%SZ'):
        if not self.date_str:
            return "1970-01-01T00:00:00Z"
        return self.format(target_fmt, len(self.date_str))

    def format(self, target_fmt, length=10):
        fmt = "%d.%m.%Y" if length == 10 else "%d%m%Y" if length == 6 else "%Y"
        return reformat_date_string(fmt, self.date_str, target_fmt)

def reformat_date_string(origin_fmt, origin_dts, target_fmt):
    return datetime.strptime(origin_dts, origin_fmt).strftime(target_fmt)


def get_date_from_iso_str(origin_fmt, origin_dts):
    return datetime.strptime(origin_fmt, origin_dts)


def get_file_ctime_iso_date_str(file_path, fmt=DT_ISO_FORMAT, wd=None):
    fp = file_path
    path = fp if wd is None else os.path.join(wd, fp)
    dt = timezone('Europe/Vienna').localize(datetime.fromtimestamp(os.path.getctime(path)).replace(microsecond=0))
    return dt.strftime(fmt)


def ts_date(fmt=TS_FORMAT):
    return datetime.fromtimestamp(time.time()).strftime(fmt)


def date_format(date, fmt=DT_ISO_FORMAT):
    return date.strftime(fmt)


def current_timestamp(fmt=DT_ISO_FMT_SEC_PREC, time_zone_id='Europe/Vienna'):
    dt = datetime.now(tz=timezone(time_zone_id))
    return dt.strftime(fmt)


def current_date(time_zone_id='Europe/Vienna'):
    return datetime.now(tz=timezone(time_zone_id))


def get_current_milli_time():
    millis = int(round(time.time() * 1000))
    return millis


def get_local_datetime_now():
    return datetime.now(tz=pytz.timezone('Europe/Vienna'))
